# -*- coding: utf-8 -*-
"""
@author: 'xiazeng'
@created: 2016/12/22 
"""
import logging
import time
import at
from at.webdriver import jsapi


logger = logging.getLogger()
MM_X5_CLASS_NAME = "com.tencent.smtt.webkit.WebView"#这个是X5的
ANDROID_WEBKIT_CLASS_NAME = "android.webkit.WebView"


class WebElementNotFound(RuntimeError):
    pass


class WebElement(object):
    def __init__(self, page, web_view_bounds=None, hid=None, css=None, text=None, cls_name=None, xpath=None, tag_name=None, name=None, text_contains=None, shadow_roots=None,
                          shadow_root=None):
        self.page = page
        self.ws_url = ""
        self.wrap_view_cls = MM_X5_CLASS_NAME
        serial = self.page.serial
        self.at = page.at
        self.web_view_rect = web_view_bounds
        #print "web_view_rect",self.web_view_rect
        self._json_element = None
        self._js_element = None
        if hid:
            word = "id"
            selector = hid
            self._selector = "#%s" % hid#根据id来进行css_selector查找
        elif css:
            word = "cssSelector"
            selector = css
            self._selector = css #css语句
        elif text:
            word = "textContent"
            selector = text
            self._selector = ':text("%s")' % text #输入文字
        elif cls_name:
            word = "className"
            selector = cls_name
            self._selector = '.%s' % text  #根据.来输入类名
        elif xpath:
            word = "xpath"
            selector = xpath
        elif tag_name:
            word = "tagName"
            selector = tag_name
            self._selector = tag_name
        elif name:
            word = "name"
            selector = name
            self._selector = name
        else:
            raise AssertionError("no way to go")
        self.word = word
        self.selector = selector

        self._instance = 0

    def instance(self, num):#相同元素中的第几个
        self._instance = num
        return self

    def word_big_first_letter(self):#转第一个字母为大写字母
        return self.word[0].upper()+self.word[1:]

    def wait_exists(self, timeout=10):#等待出现
        s = time.time()
        while time.time() - s < timeout:
            if self.exists():
                return True
            time.sleep(1)
        return False

    def screen(self, filename, quality=100):
        """
        截图，并且圈出对应的坐标点
        """
        x, y, width, height = self.rect()
        self.at.device.screen_point(filename, x, y, width, height, quality)

    def scroll_to(self, x, y):
        self.page.invoke_js(jsapi.SCROLL_TO, x, y)

    def click(self):
        if not self.page.click_by_ui:
            raise RuntimeError("not implementation")
        else:
            # self.wait_exists()
            self.find_json_element()
            if not self._js_element['in_screen']:
                self.call_js("scrollIntoView", [])
                # self.scroll_to(self._js_element['rect']['left'], self._js_element['rect']['top'])
                time.sleep(2)
                self.find_json_element(True)
            # if not self._js_element['in_screen']:
            #     raise RuntimeError(u"element invisible")
            x, y, width, height = self.rect()
            x = x + width / 2
            y = y + height / 2

            time.sleep(1)
            self.at.device.click_on_point(x, y)#点击元素的中心点
            time.sleep(1)

    def get_center_position(self):
        if not self.page.click_by_ui:
            raise RuntimeError("not implementation")
        else:
            # self.wait_exists()
            self.find_json_element()
            if not self._js_element['in_screen']:
                self.call_js("scrollIntoView", [])
                # self.scroll_to(self._js_element['rect']['left'], self._js_element['rect']['top'])
                time.sleep(2)
                self.find_json_element(True)
            if not self._js_element['in_screen']:
                raise RuntimeError(u"element invisible")
            x, y, width, height = self.rect()
            x = x + width / 2
            y = y + height / 2
            return x, y

    def long_click(self, duration=3000):
        method_name = self.word
        if not self.page.click_by_ui:
            raise RuntimeError("not implementation")
        else:
            # self.wait_exists()
            x, y, width, height = self.rect()
            x = x + width / 2
            y = y + height / 2
            time.sleep(1)
            self.at.device.long_click_on_point(x, y, duration)
            time.sleep(1)

    def web_rect(self):
        ret = self.find_json_element()
        json_rect = ret['rect']
        return json_rect["left"], json_rect['top'], json_rect["width"], json_rect["height"]

    def rect(self):
        x, y, width, height = self.web_rect()
        target_x, target_y = self.transfer_web_point(x, y)
        target_width, target_height = self.transfer_web_distance(width, height)
        return target_x, target_y, target_width, target_height

    def call_js(self, sub_method, args, timeout=10):
        r = None
        t = time.time()
        end_time = time.time()
        while end_time - t < timeout:
            try:
                r = self.page.invoke_js(jsapi.ELEMENT_OPERATION, self.selector, sub_method, args, self._instance)
                break
            except jsapi.ElementNotExists as e:
                end_time = time.time()
                if end_time - t < timeout:
                    time.sleep(2)
                else:
                    raise e
        return r

    def find_json_element(self, is_refresh=False, timeout=10):
        if self._js_element is None or is_refresh:
            self._js_element = self.call_js(None, [], timeout)
        return self._js_element

    def point_rate(self):
        rect = self.web_view_rect
        page_width = self.page.width
        page_height = self.page.height
        view_width = rect[2] - rect[0]
        view_height = rect[3] - rect[1]
        return view_width*1.0/page_width, view_height*1.0/page_height

    def transfer_web_distance(self, x, y):
        x_rate, y_rate = self.point_rate()
        return x*x_rate, y * y_rate

    def transfer_web_point(self, x, y):
        x = x if x > 0 else 0
        y = y if y > 0 else 0
        rect = self.web_view_rect
        #print "rect:",rect
        x_rate, y_rate = self.point_rate()
        target_x = int(rect[0] + x * x_rate)
        target_y = int(rect[1] + y * y_rate)
        logger.info("%d, %d" % (target_x, target_y))
        return target_x, target_y

    def exists(self, timeout=3):
        try:
            self.find_json_element(timeout=timeout)
        except:
            return False
        return True

    def scroll_to_screen(self, max_scroll_num=10):
        for i in range(max_scroll_num):
            if self.is_in_screen():
                return True
            time.sleep(1)
        raise RuntimeError(u"scroll over %d", max_scroll_num)

    def is_in_screen(self):
        """
        元素是否在屏幕内部，但是不一定可见
        """
        if not self.exists():
            return False
        json_ret = self.find_json_element(True)
        return json_ret['in_screen']

    def is_visible(self):
        '''
        元素是否在屏幕上可见
        :return:
        '''
        if not self.exists():
            return False
        json_ret = self.find_json_element(True)
        return json_ret['visible']

    def get_element(self):
        json_e = self.find_json_element()
        if json_e:
            return json_e
        else:
            raise RuntimeError(u"Not Found")

    def get_text(self):
        json_e = self.find_json_element()
        if json_e:
            return json_e.get('text')

    def enter(self, text):
        raise RuntimeError("not implementation")


    #富文本输入方法
    def innerHtml(self,text = None):
        '''
        设置content中的innerHTML
        :param text:
        :return:
        '''
        if text:
            return self.call_js(jsapi.ELEMENT_OPERATION_SUPPLY_METHOD.SET_PROPERTY,["innerHTML",text])
        else:
            return self.call_js(jsapi.ELEMENT_OPERATION_SUPPLY_METHOD.SET_PROPERTY,["innerHTML"])

    def val(self, text=None):
        """
        直接设置dom节点的value值
        :param text:
        :return:
        """
        if text:
            return self.call_js(jsapi.ELEMENT_OPERATION_SUPPLY_METHOD.SET_PROPERTY, ["value", text])
        else:
            return self.call_js(jsapi.ELEMENT_OPERATION_SUPPLY_METHOD.SET_PROPERTY, ["value"])

    def attr(self, name, value=None):
        if value:
            return self.call_js("setAttribute", [name, value])
        else:
            return self.call_js("getAttribute", [name])



#!/usr/bin/env python3

"""
Author:         xiazeng
Create time:    2021/3/11
Description:    
AtServer返回的事件
"""
import time
import websocket
import threading
import logging
import json

logger = logging.getLogger()


def run_ws_server(ws):
    ws.run_forever()


class WebSocketCli:
    def __init__(self, port):
        self.url = "ws://127.0.0.1:%d/" % port
        self.status = "init"
        self.event_callbacks = {}
        self.ensure_open()

    def add_callback(self, event: str, callback):
        callbacks = self.event_callbacks.get(event)
        if not callbacks:
            callbacks = []
            self.event_callbacks[event] = callbacks
        ids = [id(cb) for cb in callbacks]
        if id(callback) in ids:
            logger.error("event:%s, func:%s, duplicate added", id(callback), callback.__name__)
            return
        callbacks.append(callback)

    def close(self):
        self._client.close()

    def reconnect(self):
        self.close()
        self.ensure_open()

    def ensure_open(self):
        self._client = websocket.WebSocketApp(self.url, on_open=self._on_open, on_message=self._on_message,
                                              on_error=self._on_error, on_close=self._on_close)
        self._thread = threading.Thread(target=run_ws_server, args=(self._client,))
        self.status = "pending"
        self._thread.daemon = True
        self._thread.start()
        s = time.time()
        while time.time() - s < 2:
            if self.status != "pending":
                return True
        return False

    def _on_open(self):
        """
        :type ws: websocket.WebSocketApp
        """
        logger.debug('%s, %s, opened', self.url, self._thread.ident)
        self.status = "open"

    def _on_message(self, message):
        """
        :type ws: websocket.WebSocketApp
        """
        # logger.debug("%s, %s: %.1024s", self.url, self._thread.ident, message)
        msg = json.loads(message)
        callbacks = self.event_callbacks.get(msg['event'])
        if not callbacks:
            logger.debug("%s drop, no callback", msg['event'])
            return
        for callback in callbacks:
            callback(msg['data'])

    def _on_error(self, *args):
        pass
        # logger.error("%s, %s, args: %s", self.url, self._thread.ident, args)

    def _on_close(self, *args):
        self.status = "close"
        # logger.debug('%s, %s, closed in on_close, args:%s', self.url, self._thread.ident, args)


if __name__ == '__main__':
    import logging

    logging.basicConfig(level=logging.DEBUG)
    cli = WebSocketCli(9998)
    while cli.status != 'close':
        print(cli.status)
        time.sleep(1)



# -*- coding: utf-8 -*-
"""
@author: 'xiazeng'
@created: 2016/10/21 
"""
import base64
import copy
import inspect
import json
import logging
import os.path
import sys
import time
import warnings
import re

import at.core.case_repair_adapter
from at.core import uixml, javadriver, resguard, uidevice, exceptions
import at.utils.decorator as decorator

if sys.version_info[0] < 3:
    basestring = (str, unicode)
else:
    basestring = str

logger = logging.getLogger()


class AtSelector:
    SELECTOR_NIL = 0
    SELECTOR_TEXT = 1
    SELECTOR_START_TEXT = 2
    SELECTOR_CONTAINS_TEXT = 3
    SELECTOR_CLASS = 4
    SELECTOR_DESCRIPTION = 5
    SELECTOR_START_DESCRIPTION = 6
    SELECTOR_CONTAINS_DESCRIPTION = 7
    SELECTOR_INDEX = 8
    SELECTOR_INSTANCE = 9
    SELECTOR_ENABLED = 10
    SELECTOR_FOCUSED = 11
    SELECTOR_FOCUSABLE = 12
    SELECTOR_SCROLLABLE = 13
    SELECTOR_CLICKABLE = 14
    SELECTOR_CHECKED = 15
    SELECTOR_SELECTED = 16
    SELECTOR_ID = 17
    SELECTOR_PACKAGE_NAME = 18
    SELECTOR_CHILD = 19
    SELECTOR_CONTAINER = 20
    SELECTOR_PATTERN = 21
    SELECTOR_PARENT = 22
    SELECTOR_COUNT = 23
    SELECTOR_LONG_CLICKABLE = 24
    SELECTOR_TEXT_REGEX = 25
    SELECTOR_CLASS_REGEX = 26
    SELECTOR_DESCRIPTION_REGEX = 27
    SELECTOR_PACKAGE_NAME_REGEX = 28
    SELECTOR_RESOURCE_ID = 29
    SELECTOR_CHECKABLE = 30
    SELECTOR_RESOURCE_ID_REGEX = 31
    SELECTOR_USE_CUSTOM = 40

    @classmethod
    def msg(cls, sid):
        msg = {
            cls.SELECTOR_TEXT: "text",
            cls.SELECTOR_RESOURCE_ID: "rid",
            cls.SELECTOR_DESCRIPTION: "description",
            cls.SELECTOR_CLASS: "class",
            cls.SELECTOR_CONTAINS_TEXT: "contains",
            cls.SELECTOR_ENABLED: "enabled",
            cls.SELECTOR_CHECKED: "checked",
            cls.SELECTOR_CLICKABLE: "clickable",
            cls.SELECTOR_PACKAGE_NAME: "package",
            cls.SELECTOR_INSTANCE: "instance",
            cls.SELECTOR_INDEX: "index",
            cls.SELECTOR_CONTAINS_DESCRIPTION: "desc_contains",
        }
        return msg.get(sid, sid)


class Element(object):
    """控件元素，用于选择和操作控件，选择请用选择器函数，选择器函数返回自身(self)，可以做链式选择。
    非选择器函数返回None或者对应的属性
    """
    default_jd = None
    default_resguard_filename = None

    def __init__(self, **selector):
        """
        :param selector:手动填入选择器， 如果需要试用多设备，请传入jd_instance实例
        """
        self.input_interceptor = None
        self._resguard = None
        self._private_jd = None
        self._resguard_filename = None
        if "jd_instance" in selector:
            self._private_jd = selector["jd_instance"]
            del selector["jd_instance"]
        if "resguard_filename" in selector:
            self._resguard_filename = selector["resguard_filename"]
            del selector["resguard_filename"]
        self._selector = selector
        self._selectors = [{"name": "current", "values": self._selector}]

    def get_xml_attr(self):
        """获取控件所有属性

        :return: Dict 控件的xml属性
        """
        attr = {}
        for select_id, value in self.get_selector().items():
            if select_id == AtSelector.SELECTOR_PACKAGE_NAME:
                attr['package'] = value
            elif select_id == AtSelector.SELECTOR_CONTAINS_TEXT:
                logger.error("not support: %s", AtSelector.msg(select_id))
            elif select_id == AtSelector.SELECTOR_CONTAINS_DESCRIPTION:
                logger.error("not support: %s", AtSelector.msg(select_id))
            elif select_id == AtSelector.SELECTOR_TEXT:
                attr['text'] = value
            elif select_id == AtSelector.SELECTOR_CLASS:
                attr['class'] = value
            elif select_id == AtSelector.SELECTOR_RESOURCE_ID:
                attr['resource-id'] = self.resguard.retrace_id(value)
            elif select_id == AtSelector.SELECTOR_DESCRIPTION:
                attr['content-desc'] = value
            elif select_id == AtSelector.SELECTOR_ENABLED:
                attr['enabled'] = value
            else:
                logger.error("not support: %s", AtSelector.msg(select_id))
                # raise RuntimeError("unknown select_id:%s" % select_id)
        return attr

    def set_input_interceptor(self, input_interceptor):
        """操作拦截，比如拦截点击操作，用input_interceptor来实现具体的点击

        :param input_interceptor:
        :return: None
        """
        self.input_interceptor = input_interceptor

    def match_ui_view(self, ui_view):
        for select_id, value in self.get_selector().items():
            if select_id == AtSelector.SELECTOR_PACKAGE_NAME:
                if value not in ui_view.package:
                    return False
            elif select_id == AtSelector.SELECTOR_TEXT:
                if ui_view.text != value:
                    return False
            elif select_id == AtSelector.SELECTOR_START_TEXT:
                if not ui_view.text.startswith(value):
                    return False
            elif select_id == AtSelector.SELECTOR_CONTAINS_TEXT:
                if value not in ui_view.text:
                    return False
            elif select_id == AtSelector.SELECTOR_DESCRIPTION:
                if ui_view.content_desc != value:
                    return False
            elif select_id == AtSelector.SELECTOR_START_DESCRIPTION:
                if not ui_view.content_desc.startswith(value):
                    return False
            elif select_id == AtSelector.SELECTOR_CONTAINS_DESCRIPTION:
                if value not in ui_view.content_desc:
                    return False
            elif select_id == AtSelector.SELECTOR_CLASS:
                if ui_view.cls_name != value:
                    return False
            elif select_id == AtSelector.SELECTOR_RESOURCE_ID:
                if ui_view.raw_id != value:
                    return False
            elif select_id == AtSelector.SELECTOR_CHECKABLE:
                if ui_view.checkable != value:
                    return False
            elif select_id == AtSelector.SELECTOR_CHECKED:
                if ui_view.checked != value:
                    return False
            elif select_id == AtSelector.SELECTOR_INDEX:
                pass
                # index 不做校验
                # if ui_view.index != value:
                #     return False
            elif select_id == AtSelector.SELECTOR_FOCUSABLE:
                if ui_view.focusable != value:
                    return False
            elif select_id == AtSelector.SELECTOR_FOCUSED:
                if ui_view.focused != value:
                    return False
            elif select_id == AtSelector.SELECTOR_CLICKABLE:
                if ui_view.clickable != value:
                    return False
            elif select_id == AtSelector.SELECTOR_LONG_CLICKABLE:
                if ui_view.long_clickable != value:
                    return False
            elif select_id == AtSelector.SELECTOR_USE_CUSTOM:
                pass
            elif select_id == AtSelector.SELECTOR_INSTANCE:
                pass
            elif select_id == AtSelector.SELECTOR_ENABLED:
                if ui_view.enabled != value:
                    return False
            elif select_id == AtSelector.SELECTOR_TEXT_REGEX:
                if not re.match(value, ui_view.text):
                    return False
            elif select_id == AtSelector.SELECTOR_DESCRIPTION_REGEX:
                if not re.match(value, ui_view.content_desc):
                    return False
            elif select_id == AtSelector.SELECTOR_RESOURCE_ID_REGEX:
                if not re.match(value, ui_view.raw_id):
                    return False
            else:
                raise RuntimeError("unknown select_id:%s" % select_id)
        return True

    def get_selector(self):
        """返回当前选择器"""
        return self._selector

    def __repr__(self):
        values = []
        suffix = ""
        for sid, value in self._selector.items():
            if sid == AtSelector.SELECTOR_RESOURCE_ID:
                value = self.resguard.retrace_id(value)
            if sid == AtSelector.SELECTOR_PACKAGE_NAME:
                suffix = "%s=%s" % (AtSelector.msg(sid), value)
            else:
                values.append("%s=%s" % (AtSelector.msg(sid), value))
        if suffix:
            values.append(suffix)
        return u', '.join(values)

    def __hash__(self):
        return hash(repr(self))

    @property
    def jd(self):
        """
        :rtype: javadriver.JavaDriver
        :return: 
        """
        if self._private_jd:
            return self._private_jd  # type: javadriver.JavaDriver
        elif Element.default_jd:
            return Element.default_jd   # type: javadriver.JavaDriver
        else:
            raise exceptions.AtError("elem should be passed a JavaDriver")

    @decorator.cached_property
    def device(self):
        return uidevice.PyUiDevice(self.jd)

    @property
    def resguard(self):
        return resguard.Resguard.get_resguard(self.jd.serial)

    @classmethod
    def bind_java_driver(cls, jd):
        cls.default_jd = jd

    @classmethod
    def bind_resguard_filename(cls, filename):
        if filename:
            cls.default_resguard_filename = filename

    def child(self, **selector):
        """
        选择器切换，使用这个函数之后，选择器切换到孩子的方向上，后续的过滤条件只对孩子生效
        """
        self._selector = copy.deepcopy(selector)
        self._selectors.append({"name": "child", "values": self._selector})
        return self

    def parent(self, **selector):
        """
        选择器切换，使用这个函数之后，选择器切换到祖先方向上，后续的过滤条件只对祖先生效
        """
        self._selector = copy.deepcopy(selector)
        self._selectors.append({"name": "parent", "values": self._selector})
        return self

    def text(self, text=None):
        """
        选择器，过滤text=text的控件

        :param text: 完全匹配
        :return:
        """
        if text is None:
            return self._action("getText")  # self.get_attr("text")
        self._selector[AtSelector.SELECTOR_TEXT] = text
        return self

    def text_contains(self, text):
        """
        选择器，过滤包含text的控件

        :param text: 匹配包含的文本
        :return:
        """
        self._selector[AtSelector.SELECTOR_CONTAINS_TEXT] = text
        return self

    def text_start(self, text):
        """
        选择器，过滤以text开头的控件
        :param text: 匹配开始的文本
        :return:
        """
        self._selector[AtSelector.SELECTOR_START_TEXT] = text
        return self

    def instance(self, index):
        """
        选择器，符合过滤条件的元素序号，可以简单理解为从上到下，元素的顺序，0是第一个

        :param index: 按照XML的树形结构前序遍历的顺序
        :return:
        """
        self._selector[AtSelector.SELECTOR_INSTANCE] = index
        return self

    def text_matches(self, java_regx_str):
        """
        :param text: 匹配文本，请传入java的正则表达式
        :return:
        """
        self._selector[AtSelector.SELECTOR_TEXT_REGEX] = java_regx_str
        return self

    def rid(self, rid=None):
        """选择器，过滤控件的资源ID属性（resource-id）
        
        :param rid: 匹配resource-id
        :return:
        """
        if rid is None:
            return self.get_attr("resource_id")
        rid = self.resguard.resgurad_id(rid)
        self._selector[AtSelector.SELECTOR_RESOURCE_ID] = rid
        return self

    def index(self, index=None):  # 不建议使用，
        """选择器，过滤控件的index条件，可以理解为这个控件在兄弟姐妹中排行第几

        *warnings* 但是这个有时候跟渲染时机有问题，导致操作不稳定

        :param index: 匹配index，uiAutomatorviewer中的index
        :return:
        """
        warnings.warn('不建议用index来选择控件', DeprecationWarning)
        if index is None:
            return self.get_attr("index")
        self._selector[AtSelector.SELECTOR_INDEX] = index
        return self

    def clickable(self, true_or_false):
        """
        选择器，过滤控件的clickable属性
        :param true_or_false:
        :return:
        """
        self._selector[AtSelector.SELECTOR_CLICKABLE] = true_or_false
        return self

    def checkable(self, true_or_false):
        """
        选择器，过滤控件的checkable属性
        :param true_or_false:
        :return:
        """
        self._selector[AtSelector.SELECTOR_CHECKABLE] = true_or_false
        return self

    def checked(self, true_or_false):
        """选择器，过滤控件的checked属性

        :param true_or_false:
        :return:
        """
        self._selector[AtSelector.SELECTOR_CHECKED] = true_or_false
        return self

    def enabled(self, true_or_false):
        """选择器，过滤控件的enabled属性

        :param true_or_false:
        :return: self
        """
        self._selector[AtSelector.SELECTOR_ENABLED] = true_or_false
        return self

    def focusable(self, true_or_false):
        """选择器，过滤控件的focusable属性

        :param true_or_false:
        :return:
        """
        self._selector[AtSelector.SELECTOR_FOCUSABLE] = true_or_false
        return self

    def longClickable(self, true_or_false):
        """选择器，过滤控件的longClickable属性

        :param true_or_false:
        :return:
        """
        self._selector[AtSelector.SELECTOR_LONG_CLICKABLE] = true_or_false
        return self

    def selected(self, true_or_false):
        """选择器，过滤控件的select属性

        :param true_or_false:
        :return:
        """
        self._selector[AtSelector.SELECTOR_SELECTED] = true_or_false
        return self

    def use_custom(self):
        """选择器，使用at重写的过滤方法
        :return:
        """
        self._selector[AtSelector.SELECTOR_USE_CUSTOM] = 1
        return self

    def cls_name(self, cls_name=None):
        """选择器， 过滤控件的控件的类名，如：android.widget.EditText

        :param cls_name: 匹配class_name
        :return: Element
        """
        if cls_name is None:
            return self.get_attr("cls_name")
        self._selector[AtSelector.SELECTOR_CLASS] = cls_name
        return self

    def desc(self, desc, true_or_false=False):
        """选择器， 过滤控件的控件包含的description

        :param desc: description
        :param true_or_false: 找不到是否尝试滑动
        :return: Element
        """
        self.check_scroll(true_or_false)
        if desc is None:
            return self._action("getContentDescription")
        self._selector[AtSelector.SELECTOR_DESCRIPTION] = desc
        return self

    def pkg(self, pkg_name):
        """选择器，控件所属的包名

        :param pkg_name: 控件所属的包名
        :return:
        """
        self._selector[AtSelector.SELECTOR_PACKAGE_NAME] = pkg_name
        return self

    def desc_contains(self, desc, true_or_false=False):
        """选择器，选择description包含的文本

        :param desc:
        :param true_or_false:
        :return:
        """

        self.check_scroll(true_or_false)
        self._selector[AtSelector.SELECTOR_CONTAINS_DESCRIPTION] = desc
        return self

    def desc_reg(self, desc, true_or_false=False):
        self.check_scroll(true_or_false)
        self._selector[AtSelector.SELECTOR_DESCRIPTION_REGEX] = desc
        return self

    def list_view(self):
        """快捷选择器，选择ListView
        :return:
        """
        self.cls_name("android.widget.ListView")
        return self

    def edit_text(self):
        """快捷选择器， 选择EditText
        :return:
        """
        self.cls_name("android.widget.EditText")
        return self

    def get_attr(self, attr):
        views = uixml.get_views(self.jd.dump_ui(), self._selector)
        if not views:
            return None
        if len(views) == 1:
            if hasattr(views[0], attr):
                return getattr(self.views[0], attr)
            else:
                raise AttributeError("no this attribute %s in %s" % (str(attr), type(views[0])))
        else:
            rets = []
            for v in views:
                if hasattr(v, attr):
                    rets.append(getattr(v, attr))
                else:
                    raise AttributeError("no this attribute %s in %s" % (str(attr), type(v)))
            return rets

    def get_ui_views(self):
        ui_views = []
        for ui_view in self.jd.dump_ui():
            if self.match_ui_view(ui_view):
                ui_views.append(ui_view)
        return ui_views

    def _get_view(self):
        instance = self._selector.get(AtSelector.SELECTOR_INSTANCE, 0)
        for ui_view_list in self.jd.dump_all_views():
            for ui_view in ui_view_list:
                if self.match_ui_view(ui_view):
                    if instance == 0:
                        return ui_view
                    else:
                        instance -= 1
        return None

    def get_view(self, timeout=1.0):
        """获取符合条件的view

        :rtype: uixml.UiView
        :return:
        """
        s = time.time()
        retry = 2
        while time.time() - s < timeout or retry > 0:
            view = self._get_view()
            if view:
                logger.info(view)
                return view
            retry -= 1
            time.sleep(0.5)
        raise exceptions.UiNotFoundError()

    def wait_exists(self, timeout=10.0, should_throw_exception=False):
        """等待出现

        :param timeout: 超时时间
        :param should_throw_exception: 等不到是否抛异常
        :return:
        """
        s = time.time()
        if timeout == 0:
            timeout += 0.01
        while time.time() - s < timeout:
            try:
                ui_view = self.get_view(timeout=0.5)
                if ui_view:
                    logging.info("%s exists, view:%s", self, ui_view)
                    return True
            except exceptions.UiNotFoundError:
                time.sleep(0.1)
        logging.info("%s exists:%s, costs:%.2f", self, False, time.time() - s)
        if should_throw_exception:
            raise exceptions.UiNotFoundError("not exists")
        return False

    def wait_disappear(self, timeout=20, should_throw_exception=False):
        """等待当前控件消失

        :param timeout: 最大等待时间
        :param should_throw_exception:
        :return: 如果控件消失了，返回True；如果控件最后还在，返回False
        """
        s = time.time()
        while time.time() - s < timeout:
            if not self.exists(0):
                return True
            time.sleep(0.2)
        else:
            if should_throw_exception:
                raise exceptions.UiNotFoundError("still exists")
            return False

    def wait_disappear_by_idle(self, timeout, idle_time=1):
        """等待当前控件消失，会校验两次，适用于闪烁的场景，比如某个loading控件一闪一闪

        :param timeout:
        :param idle_time: 两次检查的时间间隔
        :return:
        """
        s = time.time()
        while time.time() - s < timeout:
            loop_start = time.time()
            if not self.exists(0):
                remain = time.time() - loop_start
                if remain < idle_time:
                    time.sleep(idle_time - remain)
                if not self.exists(0):
                    return True
            time.sleep(0.5)
        else:
            return False

    def get_rect(self):
        """获取矩形坐标"""
        bounds = self._action("pyGetBounds")
        return uixml.Rect(*bounds)

    def get_bounds(self):
        return self._action("pyGetBounds")

    def screen_data(self, quality=100):
        base64_str = self._action("screen", [quality])
        base64_data = base64.b64decode(base64_str)
        return base64_data

    def screen(self, filename, quality=100):
        d = self.screen_data(quality)
        open(filename, "wb").write(d)

    def get_text(self):
        """获取文本"""
        return self._action("getText")

    def get_counts(self):
        """获取匹配控件的数量"""
        if self.exists():
            return self._action("getCounts")
        return 0

    def get_desc(self):
        return self._action("getDesc")

    def sleep(self, seconds):
        time.sleep(seconds)
        return self

    def _action(self, method, params=None, is_async=False):
        """
        调用java层方法，不建议直接调用
        :param method: 方法名称
        :param params: 参数
        :param is_async: 是否用子线程
        :return:
        """
        if params is None:
            params = list()
        if is_async:
            import threading
            t = threading.Thread(target=self.jd.request_ui_selector, name="", args=(self._selectors, method, params))
            t.setDaemon(True)
            t.start()
        else:
            return self.jd.request_ui_selector(self._selectors, method, params)

    def record(self, name):
        if self.input_interceptor:
            self.jd.push_op(u"%s: %s", name, self, display=self.input_interceptor.display_id)
        else:
            self.jd.push_op(u"%s: %s", name, self)

    def check_scroll(self, is_scroll):
        if is_scroll:
            if self.exists(0):
                return True
            if not self._action("scrollExists"):
                logger.error("scrollExists failed")
            else:
                return True
        return False

    def scroll_exists(self):
        return self.check_scroll(True)

    def fast_click(self, is_scroll=False):
        self.record(u"快速点击")
        return self.long_click(100, is_scroll=is_scroll)

    def drag_to(self, x, y, steps=120, is_scroll=False, is_async=False):
        self.record(u"拖动")
        self.check_scroll(is_scroll)
        self._action("dragTo", [int(x), int(y), steps], is_async=is_async)

    def click_position(self,is_scroll=False, is_retry=False, is_url=False):
        self.check_scroll(is_scroll)
        ret = self._action("click")
        # ret = False
        if ret is False:
            logger.warning("click failed, %s, is_retry %s", ret, is_retry)
            if is_retry:
                try:
                    bounds = self.get_bounds()
                    x = (bounds[0] + bounds[2]) / 8
                    y = (bounds[1] + bounds[3]) / 5
                    self.jd.adb.click_point(x, y)
                except:
                    logger.exception("retry click failed")
            if is_url:
                bounds = self.get_bounds()
                width = bounds[2] - bounds[0]
                height = bounds[3] - bounds[1]
                x = bounds[0] + width / 2
                y = bounds[1] + height / 8
                device = uidevice.PyUiDevice(self.jd)
                time.sleep(2)
                if not device.click_on_point(x, y):
                    y += height / 8
                    time.sleep(2)
                    return device.click_on_point(x, y)
                else:
                    return True
        return ret

    @at.core.case_repair_adapter.deco_do_action
    def click(self, is_scroll=False, is_retry=False, is_url=False):
        """点击

        :param is_url:
        :param is_retry:
        :param is_scroll: 不存在的时候是否滑动
        :return:
        """
        self.record(u"点击")
        self.check_scroll(is_scroll)
        if self.input_interceptor:
            view = self.get_view(10)
            logger.debug(view)
            return self.input_interceptor.click(view.center_x, view.center_y)
        if is_url:
            bounds = self.get_bounds()
            width = bounds[2] - bounds[0]
            height = bounds[3] - bounds[1]
            x = bounds[0] + width / 5
            device = uidevice.PyUiDevice(self.jd)
            retrys = 3
            while retrys > 0:
                y = bounds[1] + height*9.0/10 / retrys   # 尝试3次分别点不同地方
                device.click_on_point(x, y)
                capture_op = self.jd.get_capture_op()
                self.jd.set_capture_op(False)
                if self.wait_disappear(5):
                    self.jd.set_capture_op(capture_op)
                    return True
                self.jd.set_capture_op(capture_op)
                retrys = retrys-1
            return False

        ret = self._action("click")
        if ret is False:
            logger.warning("click failed, %s, is_retry %s", ret, is_retry)
            if is_retry:
                try:
                    bounds = self.get_bounds()
                    x = (bounds[0] + bounds[2])/2
                    y = (bounds[1] + bounds[3])/2
                    self.jd.adb.click_point(x, y)
                except:
                    logger.exception("retry click failed")
        return ret

    def swipe_left(self, steps=50):
        self.record(u"向左滑动")
        return self._action("swipeLeft", [steps])

    def swipe_up(self, steps=50):
        self.record(u"向上滑动")
        return self._action("swipeUp", [steps])

    def swipe_right(self, steps=50):
        self.record(u"向右滑动")
        return self._action("swipeRight", [steps])

    def swipe_down(self, steps=50):
        self.record(u"向下滑动")
        return self._action("swipeDown", [steps])

    def delay_click(self, delay):
        """
        延后delay秒点击
        :param delay:
        :return:
        """
        self.record(u"延迟点击(%ss):" % delay)
        return self._action("delayClick", [delay * 1000])

    @at.core.case_repair_adapter.deco_do_action
    def double_click(self, is_scroll=False):
        """双击"""
        self.record(u"双击")
        self.check_scroll(is_scroll)
        if self.input_interceptor:
            rect = self.get_rect()
            return self.input_interceptor.double_click(rect.center_x, rect.center_y)
        else:
            return self._action("doubleClick", [100])

    @at.core.case_repair_adapter.deco_do_action
    def long_click(self,  duration=4000, is_scroll=False):
        """长按"""
        self.record(u"长按")
        self.check_scroll(is_scroll)
        if self.input_interceptor:
            rect = self.get_rect()
            return self.input_interceptor.long_click(rect.center_x, rect.center_y)
        else:
            return self._action("longPress", [duration, ])

    def click_if_exists(self, timeout=3):
        """如果存在就点击，不存在就不操作，不抛出异常"""
        if timeout == 0:
            try:
                view = self.get_view()
                self.device.click_on_ui_view(view)
                return True
            except exceptions.UiNotFoundError:
                logger.warning("not exists by selector:" + self.__repr__())
                return False
        else:
            t = time.time()
            while time.time() - t < timeout:
                if self.exists(0.5):
                    try:
                        # 可能真正点击的时候又不在了，所以这个异常应该要catch住
                        return self.click()
                    except:
                        return False
            else:
                logger.warning("not exists by selector:" + self.__repr__())
                return False

    def click_button_right(self, is_scroll=False):
        self.record(u"点击右下角")
        self.check_scroll(is_scroll)
        return self._action("clickBottomRight")

    @at.core.case_repair_adapter.deco_do_action
    def enter(self, text, is_clear_text=False, is_scroll=False, is_click=True):
        """输入文本

        :param text:输入的内容
        :param is_clear_text:
        :param is_scroll:
        :param is_click: 输入前是否需要点击，小程序的组件需要传False，并且先调用click方法点击触发输入光标在输入框
        :return:
        """
        self.record(u"输入:%s" % text)
        self.check_scroll(is_scroll)
        if not isinstance(text, basestring):
            raise TypeError(u"需要传入字符串，而实际类型是：%s" % str(type(text)))
        logger.info("enter %s %s, %s" % (text, str(self._selector), is_clear_text))
        return self._action("setText", [text, is_clear_text, is_click])

    def enter_chinese(self, text, is_clear_text=False, is_scroll=False, is_click=True):
        warnings.warn('enter_chinese', DeprecationWarning)
        self.record(u"输(%s)" % text)
        self.check_scroll(is_scroll)
        if not isinstance(text, basestring):
            raise TypeError(u"需要传入字符串，而实际类型是：%s" % str(type(text)))
        logger.info("enter %s %s" % (text, str(self._selector)))
        return self._action("setChineseText", [text, is_clear_text, is_click])

    def exists(self, seconds=2.000):  # seconds指一个最小时间，而不是最大时间，最大时间无法控制
        """是否存在"""
        # self.record(u"exists")
        return self.wait_exists(seconds)

    def tap_scroll(self, direction="up"):
        rect = self.get_bounds()
        device = uidevice.PyUiDevice(self.jd)
        center_x = (rect[0]+rect[2])/2
        center_y = (rect[1]+rect[3])/2
        min_width = rect[0] + int((rect[2] - rect[0])*1.0/4)
        max_width = rect[0] + int((rect[2] - rect[0])*3.0/4)
        min_height = rect[1] + int((rect[3] - rect[1])*1.0/4)
        max_height = rect[1] + int((rect[3] - rect[1])*3.0/4)
        duration = 100
        steps = 2
        if direction == "up":
            device.fast_scroll(center_x, max_height, center_x, min_height, duration, steps)
        elif direction == "down":
            device.fast_scroll(center_x, min_height, center_x, max_height, duration, steps)
        elif direction == "left":
            device.fast_scroll(max_width, center_y, min_width, center_y, duration, steps)
        elif direction == "right":
            device.fast_scroll(min_width, center_y, max_width, center_y, duration, steps)
        else:
            raise RuntimeError("direction should be one of up, down, left, right")

    def assert_exists(self, timeout=10):
        """动作函数，校验控件是否存在，不存在抛出Assert异常

        :param timeout:
        :return:
        """
        s = time.time()
        while time.time() - s < timeout:
            if self.exists(0.5):
                return True
            time.sleep(0.5)
        raise exceptions.UiNotFoundError("not exists")

    def assert_not_exist(self, timeout=3):
        """校验控件是否存在，存在抛出Assert异常

        :param timeout:
        :return:
        """
        s = time.time()
        exists = True
        while time.time() - s < timeout:
            try:
                if self.get_view():
                    exists = True
            except exceptions.UiNotFoundError:
                exists = False
        if exists:
            raise AssertionError("%s not exists" % self)



#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         lockerzhang
Filename:       logcolor.py
Create time:    2019-08-28 17:31
Description:

"""

import logging
import re

logger = logging.getLogger("minium")
logger.propagate = False
LOG_FORMATTER = "%(levelname)-5.5s %(asctime)s %(filename)-10s %(funcName)-15s %(lineno)-3d %(message)s"

console_handler = logging.StreamHandler()
formatter = logging.Formatter(LOG_FORMATTER)
console_handler.setFormatter(formatter)
logger.addHandler(console_handler)
logger.setLevel(logging.DEBUG)
g_console_handler = console_handler


# now we patch Python code to add color support to logging.StreamHandler
class CustomWindowsFilter(logging.Filterer):
    def _set_color(self, code):
        import ctypes

        # Constants from the Windows API
        self.STD_OUTPUT_HANDLE = -11
        hdl = ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
        ctypes.windll.kernel32.SetConsoleTextAttribute(hdl, code)

    def filter(self, record):
        FOREGROUND_BLUE = 0x0001  # text color contains blue.
        FOREGROUND_GREEN = 0x0002  # text color contains green.
        FOREGROUND_RED = 0x0004  # text color contains red.
        FOREGROUND_INTENSITY = 0x0008  # text color is intensified.
        FOREGROUND_WHITE = FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED
        # winbase.h
        STD_INPUT_HANDLE = -10
        STD_OUTPUT_HANDLE = -11
        STD_ERROR_HANDLE = -12

        # wincon.h
        FOREGROUND_BLACK = 0x0000
        FOREGROUND_BLUE = 0x0001
        FOREGROUND_GREEN = 0x0002
        FOREGROUND_CYAN = 0x0003
        FOREGROUND_RED = 0x0004
        FOREGROUND_MAGENTA = 0x0005
        FOREGROUND_YELLOW = 0x0006
        FOREGROUND_GREY = 0x0007
        FOREGROUND_INTENSITY = 0x0008  # foreground color is intensified.

        BACKGROUND_BLACK = 0x0000
        BACKGROUND_BLUE = 0x0010
        BACKGROUND_GREEN = 0x0020
        BACKGROUND_CYAN = 0x0030
        BACKGROUND_RED = 0x0040
        BACKGROUND_MAGENTA = 0x0050
        BACKGROUND_YELLOW = 0x0060
        BACKGROUND_GREY = 0x0070
        BACKGROUND_INTENSITY = 0x0080  # background color is intensified.
        levelno = record.levelno

        if levelno >= 50:
            color = (
                BACKGROUND_YELLOW
                | FOREGROUND_RED
                | FOREGROUND_INTENSITY
                | BACKGROUND_INTENSITY
            )
        elif levelno >= 40:
            color = FOREGROUND_RED | FOREGROUND_INTENSITY
        elif levelno >= 30:
            color = FOREGROUND_YELLOW | FOREGROUND_INTENSITY
        elif levelno >= 20:
            color = FOREGROUND_GREEN
        elif levelno >= 10:
            color = FOREGROUND_MAGENTA
        else:
            color = FOREGROUND_WHITE
        self._set_color(color)

        self._set_color(FOREGROUND_WHITE)
        return record


class CustomNoFilter(logging.Filterer):
    def filter(self, record):
        reg_1 = r"\x1b\[\d+m"
        try:
            while True:
                comp_1 = re.search(reg_1, record.msg)
                if comp_1:
                    record.msg = record.msg.replace(comp_1.group(0), "")
                else:
                    break
        except Exception as e:
            pass
        return record


class CustomKeywordFilter(logging.Filterer):
    """
    过滤系统相关的log，用户一般不关注
    """

    FILTER_MAP = {
        "App.bindingCalled": "mini_send_request|mini_request_callback",
        "App.logAdded": "",
    }

    def filter(self, record):
        if isinstance(record.msg, (str, bytes)):
            for key, value in CustomKeywordFilter.FILTER_MAP.items():
                if value:
                    if record.msg.find(key) >= 0 and re.match(
                        ".*(%s).*" % value, record.msg
                    ):
                        return 0
                else:
                    if record.msg.find(key) >= 0:
                        return 0
        return record


class CustomFileFilter(logging.Filterer):
    FILTER_FILE = {
        "javadriver.py": "",
        "adbwrap.py": "",
        "basedriver.py": "",
        "apkinfo.py": "",
        "command_line.py": "",
    }

    def filter(self, record: logging.LogRecord) -> bool:
        if record.filename in CustomFileFilter.FILTER_FILE:
            if not CustomFileFilter.FILTER_FILE[record.filename]:
                return 0
            elif CustomFileFilter.FILTER_FILE[record.filename] == record.funcName:
                return 0
        return super().filter(record)


class CustomMacFilter(logging.Filterer):
    def filter(self, record):
        levelno = record.levelno
        if levelno >= 50:
            color = "\x1b[31m"  # red
        elif levelno >= 40:
            color = "\x1b[31m"  # red
        elif levelno >= 30:
            color = "\x1b[33m"  # yellow
        elif levelno >= 20:
            color = "\x1b[32m"  # green
        elif levelno >= 10:
            color = "\x1b[35m"  # pink
        else:
            color = "\x1b[0m"  # normal
        try:
            record.msg = color + record.msg + "\x1b[0m"  # normal
        except Exception as e:
            pass
        # print "after"
        return record


#
# def add_coloring_to_emit_windows(fn):
#     # add methods we need to the class
#     def _out_handle(self):
#         import ctypes
#
#         return ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
#
#     out_handle = property(_out_handle)
#
#
#
#     setattr(logging.StreamHandler, "_set_color", _set_color)
#
#     def new(*args):
#         FOREGROUND_BLUE = 0x0001  # text color contains blue.
#         FOREGROUND_GREEN = 0x0002  # text color contains green.
#         FOREGROUND_RED = 0x0004  # text color contains red.
#         FOREGROUND_INTENSITY = 0x0008  # text color is intensified.
#         FOREGROUND_WHITE = FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED
#         # winbase.h
#         STD_INPUT_HANDLE = -10
#         STD_OUTPUT_HANDLE = -11
#         STD_ERROR_HANDLE = -12
#
#         # wincon.h
#         FOREGROUND_BLACK = 0x0000
#         FOREGROUND_BLUE = 0x0001
#         FOREGROUND_GREEN = 0x0002
#         FOREGROUND_CYAN = 0x0003
#         FOREGROUND_RED = 0x0004
#         FOREGROUND_MAGENTA = 0x0005
#         FOREGROUND_YELLOW = 0x0006
#         FOREGROUND_GREY = 0x0007
#         FOREGROUND_INTENSITY = 0x0008  # foreground color is intensified.
#
#         BACKGROUND_BLACK = 0x0000
#         BACKGROUND_BLUE = 0x0010
#         BACKGROUND_GREEN = 0x0020
#         BACKGROUND_CYAN = 0x0030
#         BACKGROUND_RED = 0x0040
#         BACKGROUND_MAGENTA = 0x0050
#         BACKGROUND_YELLOW = 0x0060
#         BACKGROUND_GREY = 0x0070
#         BACKGROUND_INTENSITY = 0x0080  # background color is intensified.
#
#         levelno = args[1].levelno
#         if levelno >= 50:
#             color = (
#                 BACKGROUND_YELLOW
#                 | FOREGROUND_RED
#                 | FOREGROUND_INTENSITY
#                 | BACKGROUND_INTENSITY
#             )
#         elif levelno >= 40:
#             color = FOREGROUND_RED | FOREGROUND_INTENSITY
#         elif levelno >= 30:
#             color = FOREGROUND_YELLOW | FOREGROUND_INTENSITY
#         elif levelno >= 20:
#             color = FOREGROUND_GREEN
#         elif levelno >= 10:
#             color = FOREGROUND_MAGENTA
#         else:
#             color = FOREGROUND_WHITE
#         args[0]._set_color(color)
#
#         ret = fn(*args)
#         args[0]._set_color(FOREGROUND_WHITE)
#         # print "after"
#         return ret
#
#     return new
#
#
# def add_coloring_to_emit_ansi(fn,*args):
#     # add methods we need to the class
#     # def new(*args):
#         levelno = args[1].levelno
#         if levelno >= 50:
#             color = "\x1b[31m"  # red
#         elif levelno >= 40:
#             color = "\x1b[31m"  # red
#         elif levelno >= 30:
#             color = "\x1b[33m"  # yellow
#         elif levelno >= 20:
#             color = "\x1b[32m"  # green
#         elif levelno >= 10:
#             color = "\x1b[35m"  # pink
#         else:
#             color = "\x1b[0m"  # normal
#         try:
#             args[1].msg = color + args[1].msg + "\x1b[0m"  # normal
#         except Exception as e:
#             pass
#         # print "after"
#         return fn(*args)

# return new

# def del_coloring_to_emit_ansi(fn):
#     def new()

# import platform
#
# if platform.system() == "Windows":
#     # Windows does not support ANSI escapes and we are using API calls to set the console color
#     logging.StreamHandler.emit = add_coloring_to_emit_windows(
#         logging.StreamHandler.emit
#     )
# else:
#     # all non-Windows platforms are supporting ANSI escapes so we use them
#     logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)

global colorfilter
colorfilter = CustomMacFilter()
global nofilter
nofilter = CustomNoFilter()
global kwfilter
kwfilter = CustomKeywordFilter()
global filefilter
filefilter = CustomFileFilter()
console_handler.addFilter(colorfilter)

if __name__ == "__main__":
    import logging.handlers

    logger = logging.getLogger("yopo")
    LOG_FORMATTER = "%(levelname)-5.5s %(asctime)s %(filename)-10s %(funcName)-15s %(lineno)-3d %(message)s"
    FILENAME_LOGGER = "yopotest.log"
    # TO SCREEN
    console_handler = logging.StreamHandler()
    console_handler.addFilter(colorfilter)
    formatter = logging.Formatter(LOG_FORMATTER)
    console_handler.setFormatter(formatter)
    console_handler.setLevel(logging.INFO)
    logger.addHandler(console_handler)
    logger.setLevel(logging.DEBUG)
    # TO FILE
    log_path = FILENAME_LOGGER
    case_log_handler = logging.handlers.RotatingFileHandler(
        log_path, maxBytes=1024 * 1024, encoding="utf-8"
    )
    case_log_handler.addFilter(nofilter)
    formatter = logging.Formatter(LOG_FORMATTER)
    case_log_handler.setFormatter(formatter)
    case_log_handler.setLevel(logging.DEBUG)
    logger.addHandler(case_log_handler)
    logger.setLevel(logging.DEBUG)

    logger.debug("debug msg")
    logger.info("info msg")
    logger.warning("warn msg")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@Author: lockerzhang
@LastEditors: lockerzhang
@Description: 自动化 session
@Date: 2019-03-11 14:42:37
@LastEditTime: 2019-03-11 21:05:21
"""
import minium
import logging.handlers
import json
import os
import time
from minium.framework import report
from multiprocessing import Process, Queue
from queue import Empty
from minium.framework import minitest
from .miniconfig import MiniConfig
import minium.miniprogram.base_driver.minium_log as minum_log
import traceback
import sys

logger = logging.getLogger("minium.SESSION")
FILENAME_SUMMARY = "summary.json"


class Session(Process):
    """
    管理多机多账号运行
    """

    def __init__(self, conf: dict, queue: Queue, generate_report: bool = False):
        """
        初始化
        :param conf: 配置
        :param queue: 用例队列
        :param generate_report: 是否生成报告
        """
        super(Session, self).__init__()
        self.port = conf.get("port")
        self.nick_name = conf.get("account_info", {}).get("wx_nick_name", None)
        self.open_id = conf.get("account_info", {}).get("open_id", None)
        self.device_id = conf.get("device_desire", {}).get("device_info", {}).get("udid", None)
        if self.device_id is None:
            self.device_id = (
                conf.get("device_desire", {}).get("device_info", {}).get("serial", None)
            )
        self.os_type = conf.get("platform")
        self.platform = conf.get("app")
        # bug 预警: 多线程中传入不能被序列化的类时，在 spawn 模式下会导致线程启动失败
        self.conf = json.dumps(conf)
        self.queue = queue
        self.mini = None
        self.native = None
        self.generate_report = generate_report

    def run(self) -> None:
        minitest.AssertBase.CONFIG = MiniConfig(json.loads(self.conf))
        minitest.AssertBase.setUpConfig()
        while True:
            try:
                tests = self.queue.get(timeout=10)
                result = self.run_case(tests=tests)
                result.dumps(minitest.AssertBase.CONFIG.outputs)
            except Empty:
                logger.info(f"case queue is empty, test complete, release {self.device_id}")
                break
            except Exception as e:
                logger.info("run case error")
                logger.exception(e)
                exc_type, exc_value, exc_traceback = sys.exc_info()
                minum_log.report_exception(minum_log.ExceptionData(exc_value))
                break

        try:
            if not minitest.AssertBase.CONFIG.debug:
                minitest.full_reset()
        except Exception as e:
            logger.error("reset error")
            logger.exception(e)

        # gen report
        if self.generate_report:
            report.imp_main(minitest.AssertBase.CONFIG.outputs)

    @staticmethod
    def run_case(tests):
        """
        执行用例
        :return:
        """
        result = minium.MiniResult()
        tests.run(result)
        result.print_shot_msg()
        return result



# -*- coding: utf-8 -*-
# Created by xiazeng on 2020/4/7
import logging
import at
import time
import sys
import at.core.adbwrap
from at.core.config import *


def get_adb(serial):
    return at.core.adbwrap.AdbWrap.apply_adb(serial)


def check_at_env(serial=None):
    warning_list = []
    adb = get_adb(serial)
    if not adb.check_installed_apk(TEST_APP_PKG, STUB_APK_PATH, True):
        warning_list.append(u"请手动安装: %s" % STUB_APK_PATH)
    if not warning_list:
        logging.info("环境检查OK!")
    else:
        logging.info("运行环境有问题:")
        for warning_line in warning_list:
            logging.info(warning_line)


def on_click(node):
    # line = f"self.at.e.pkg('{node['package']}')"
    prefix = f"self.e"
    selector = ""
    if node['content-desc']:
        selector += f".desc('{node['content-desc']}')"
    if node['resource-id']:
        selector += f".rid('{node['resource-id']}')"
    if node['text']:
        selector += f".text('{node['text']}')"
    if not selector:
        print("生成命令失败，请自己组装：", node)
        return

    print(prefix + selector + ".click()")


def install(serial, path):
    a = at.At(serial)   # at会检测弹框
    if not a.adb.install(path):
        logging.info("install failed:%s", path)
        sys.exit(-1)


def dump(serial):
    a = at.At(serial)
    index = 0
    for ui_views in a.java_driver.dump_all_views():
        logging.info(f"{'='*40}windows {index}{'='*40}")
        for ui_view in ui_views:
            logging.info(ui_view)
        index += 1


def main():
    logging.basicConfig(level=logging.DEBUG, format='%(message)s')
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--version', dest='version', default=None, help='print build info', action="store_true")
    parser.add_argument('-s', '--serial', dest='serial', default=None, help='android test helper')
    parser.add_argument('-i', '--install', dest='install', default=None, help='install apk')
    parser.add_argument('--check', dest='check_env', help='check wechat-at env', action="store_true")
    parser.add_argument('-e', dest='event', help='logcat输出点击事件', action="store_true")
    parser.add_argument('-d', '--dump', dest='dump', default=None, action="store_true")
    args = parser.parse_args()

    try:
        import argcomplete
        argcomplete.autocomplete(parser)
    except ImportError:
        logging.error("failed import argcomplete")
    serial = args.serial
    if args.check_env:
        check_at_env(serial)
    elif args.version:
        logging.info(at.build_info())
        sys.exit(0)
    elif args.event:
        a = at.At(serial)
        try:
            a.on_at_event("click", on_click)
            print("请开始操作手机:")
            s = time.time()
            while time.time() - s < 10 * 60 * 60:
                time.sleep(1)
                if not a.java_driver.is_remote_running():
                    break
        except Exception:
            a.release()
    elif args.install:
        install(serial, args.install)
    elif args.dump:
        dump(serial)
    else:
        parser.print_help()


if __name__ == '__main__':
    main()



import os.path
import sys

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "libs"))
sys.path.append(os.path.join(os.path.dirname(__file__)))
import json
import argparse
import tools.report_issue
import requests

work_root = os.path.abspath(".")

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-k",
        "--kao",
        dest="upload_failed_case",
        action="store_true",
        default=False,
        help="上传最近失败用例的日志和代码，截图",
    )
    parser.add_argument(
        "-i",
        dest="sync_image",
        action="store_true",
        default=False,
        help="上传最近失败用例的日志和代码，截图",
    )
    parser.add_argument("-m", "--msg", dest="msg", default=None)
    parser.add_argument("-t", "--token", dest="token", default=None)
    parser.add_argument("-a", "--assigns", dest="assigns", default=None)
    parser.add_argument("-id", "--proid", dest="proid", default=None)
    parser_args = parser.parse_args()
    upload_failed_case = parser_args.upload_failed_case
    msg = parser_args.msg
    token = parser_args.token
    assigns = parser_args.assigns
    proid = parser_args.proid

    if upload_failed_case:
        # config_path = os.path.join(work_root, ".git.json")
        last_failed_path = os.path.join(work_root, "outputs", "meta.json")
        # if not os.path.exists(config_path):
        #     print("%s not exists" % config_path)
        #     return
        if not os.path.exists(last_failed_path):
            print("%s not exists", last_failed_path)
        if not msg:
            msg = input("输入问题简述（回车表示无）: ")
        if not proid:
            proid = int(input("输入project ID："))
        else:
            proid = int(proid)
        if assigns:
            assigns = assigns.split(",")
        if not token:
            token = input("输入个人设置中的私人令牌：")
        # git_config = json.load(open(config_path))
        last_failed_cases = json.load(open(last_failed_path))
        # assigns = ["xxx"]
        if "task_datas" not in last_failed_cases:
            return
        if len(last_failed_cases["task_datas"]) < 1:
            return
        if "case_datas" not in last_failed_cases["task_datas"][0]:
            return
        for case_data_item in last_failed_cases["task_datas"][0]["case_datas"]:
            result_filepath = os.path.join(
                work_root,
                "outputs",
                case_data_item["relative"],
                case_data_item["filename"],
            )
            url = tools.report_issue.upload_issue(token, proid, result_filepath, msg, assigns)
            if url != "caseissuccess":
                print(url)
            #     session = requests.Session()
            #     session.trust_env = False
            #     r = session.post(
            #         robot_url,
            #         data=json.dumps(
            #             {
            #                 "msgtype": "text",
            #                 "mentioned_list": assigns,
            #                 "text": {
            #                     "content": f"{git_config['user']}提交了issue:\n{msg}\n请查看: {url}"
            #                 },
            #             }
            #         ),
            #     )
            #     print(r.text)
            #     r.raise_for_status()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Created by xiazeng on 2020/4/13
import logging
import time
from attestbase import AtTestBase

logger = logging.getLogger()


class AtEvent(AtTestBase):
    def on_click(self, data):
        print(data)

    def test_a(self):
        self.at.on_at_event("click", self.on_click)
        time.sleep(120)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import time
import platform
import os
import json
import subprocess
from .base_driver.app import App
from .base_driver.connection import Connection
from minium.framework.exception import *
from .base_driver.minium_object import MiniumObject

MAC_DEVTOOL_PATH = "/Applications/QQ小程序开发者工具.app/Contents/MacOS/cli"
WINDOWS_DEVTOOL_PATH = "path/to/devTool/cli.bat"
TEST_PORT = "9421"
# cur_path = os.path.dirname(os.path.realpath(__file__))
# resource_path = os.path.join(os.path.dirname(cur_path), "qq_resources")
# if not os.path.exists(resource_path):
#     os.mkdir(resource_path)


class LogLevel(object):
    INFO = 20
    DEBUG_SEND = 12
    METHOD_TRACE = 11
    DEBUG = 9


class QQMinium(MiniumObject):
    def __init__(self, conf, uri="ws://localhost", **kwargs):
        super().__init__()
        self.logger.info("init qq minium ...")
        self.sessions = {}
        self.app = None
        self.connection = None
        test_port = str(conf.test_port) if conf.get("test_port", None) else str(TEST_PORT)
        self.uri = uri + ":" + test_port
        self.project_path = conf.project_path
        self.test_port = test_port
        if conf.get("dev_tool_path", None):
            self.dev_tool_path = conf.dev_tool_path
        elif "Darwin" in platform.platform():
            self.dev_tool_path = MAC_DEVTOOL_PATH
        elif "Windows" in platform.platform():
            self.dev_tool_path = WINDOWS_DEVTOOL_PATH
        else:
            self.logger.warning("Dev tool doesn't support current OS yet")
        self._is_windows = "Windows" in platform.platform()
        self.launch_dev_tool()

    def launch_dev_tool(self):
        self.logger.info("Starting dev tool ...")
        if self.project_path:
            if not os.path.exists(self.dev_tool_path):
                raise MiniConfigError("dev_tool_path: %s not exists" % self.dev_tool_path)
            if not os.path.exists(self.project_path):
                raise MiniConfigError("project_path: %s not exists" % self.project_path)
            if not os.path.isdir(self.project_path):
                raise MiniConfigError("project_path: %s is not director" % self.project_path)
            if self._is_windows:
                cmd_template = '"%s" --auto %s --auto-port %s'
            else:
                cmd_template = "%s --auto %s --auto-port %s"
            cmd_start = cmd_template % (
                self.dev_tool_path,
                self.project_path,
                self.test_port,
            )
            status = self._do_shell(cmd_start, True)
            if self._is_windows:
                time.sleep(10)
            else:
                time.sleep(5)
            if status[-1] == "":
                self.logger.info("Dev tool is opening, but can't open project, quit now...")
                cmd = self.dev_tool_path + " --quit"
                self._do_shell(cmd, True)
                time.sleep(3)
                self.logger.info("Starting dev tool again...")
                status = self._do_shell(cmd_start, True)
                Exception("Please check MiniProgram project if config error or not") if status[
                    -1
                ] == "" else self.logger.info("Restart finish")
        else:
            status = ["Open project with automation enabled success  /Users/sherlock/svn/demo"]
        if "Open project with automation enabled success " in status[-1]:
            try:
                connection = Connection(self.uri)
                self.app = App(connection)
                self.connection = connection
            except Exception as e:
                import traceback

                traceback.print_exc()
                self.logger.exception(str(e))
                exit(0)
        else:
            self.logger.warning(
                "Open project with automation enabled fail! Try to login dev tool ..."
            )
            self.launch_dev_tool_with_login()

    def launch_dev_tool_with_login(self):
        if self._is_windows:
            cmd_template = '"%s" -l --login-qr-output terminal --auto --auto %s --auto-port %s'
        else:
            cmd_template = "%s  -l --login-qr-output terminal --auto --auto %s --auto-port %s"
        cmd_start = cmd_template % (
            self.dev_tool_path,
            self.project_path,
            self.test_port,
        )
        status = self._do_shell(cmd_start, True)
        if "Open project with automation enabled success " in status[-1]:
            connection = Connection(self.uri)
            self.app = App(connection)
            self.connection = connection
        else:
            self.logger.error(
                "Open project with automation enabled fail! make sure project path is right"
            )
            exit(-1)

    def get_app_json(self):
        if not self.project_path:
            return None
        try:
            paths = [
                os.path.join(self.project_path, "app.json"),
                os.path.join(self.project_path, "miniprogram", "app.json"),
            ]
            for path in paths:
                if os.path.exists(path):
                    return json.load(open(path, "rb"))
            raise FileNotFoundError()
        except Exception as e:
            self.logger.error("Get app json fail, cause: %s" % str(e))
            return None

    def get_system_info(self):
        return self.app.call_wx_method("getSystemInfoSync").result.result

    def enable_remote_debug(self, path=None, use_push=None, connect_timeout=None):
        if path is None:
            path = os.path.join("", "debug_qrcode.jpg")
        qrUrl = self.connection.send("Tool.enableRemoteDebug", max_timeout=180).result.qrUrl
        self.logger.debug("Tool.enableRemoteDebug  " + qrUrl)
        return qrUrl

    def shutdown(self):
        self.logger.info("QQ Dev tool closing")
        # if not os.path.exists(self.project_path):
        #     self.logger.error("dev_tool_path: %s not exists" % self.dev_tool_path)
        #     return
        if self._is_windows:
            status, ret = subprocess.getstatusoutput('"{self.dev_tool_path}" --quit')
            # status, ret = subprocess.getstatusoutput(f'"{self.dev_tool_path}" --quit')
        else:
            status, ret = subprocess.getstatusoutput("{self.dev_tool_path} --quit")
            # status, ret = subprocess.getstatusoutput(f'{self.dev_tool_path} --quit')

        if status == 0:
            self.logger.info("QQ Dev tool has quit, Minium Test complete")
        else:
            self.logger.error(ret)
            self.logger.warning("QQ Minium Test complete, but dev tool has not quit yet")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@Author: lockerzhang
@LastEditors: lockerzhang
@Description: client 入口
@Date: 2019-03-11 14:42:52
@LastEditTime: 2019-06-05 15:05:04
"""
import platform
import os
import base64
import json
import re
import threading
import time
from websocket import WebSocketConnectionClosedException

from minium.utils.injectjs import getInjectJsCode, setInjectJsMode
from .base_driver.version import build_version
from .base_driver.minium import BaseMinium
from .base_driver.app import App
from .base_driver.connection import Connection
from .base_driver.minium_object import MiniumObject
from ..framework.miniconfig import MiniConfig, get_log_level
from ..framework.exception import *
from ..native import get_native_driver, BaseNative
from ..utils.platforms import *
from ..utils.utils import (
    isWindows,
    isMacOS,
    Version,
    WaitThread,
    WaitTimeoutError,
    add_path_to_env,
    pick_unuse_port,
    catch,
)
from ..utils.emitter import ee

MAC_DEVTOOL_PATH = "/Applications/wechatwebdevtools.app/Contents/MacOS/cli"
WINDOWS_DEVTOOL_PATH = "C:/Program Files (x86)/Tencent/微信web开发者工具/cli.bat"
TEST_PORT = 9420
UPLOAD_URL = "https://stream.weixin.qq.com/weapp/UploadFile"


class LogLevel(object):
    INFO = 20
    DEBUG_SEND = 12
    METHOD_TRACE = 11
    DEBUG = 9


class WXMinium(BaseMinium):
    """
    自动化入口
    """

    def __init__(
        self,
        conf: MiniConfig = None,
        uri="ws://localhost",
        native: BaseNative = None,
        **kwargs,
    ):
        """
        初始化
        :param uri: WebSocket 地址
        :param conf: 配置
        :param native: native实例
        """
        super().__init__()
        # 私有变量
        self.__wait_for_initialized_time = None  # 等待app.initialized信号的最后时间
        # 公有变量
        if not conf:
            conf = MiniConfig()
        elif isinstance(conf, dict):
            conf = MiniConfig(conf)
        self.conf = conf
        self.logger.setLevel(get_log_level(conf.debug_mode))
        self.native = native
        if native is None and conf.platform:
            try:
                self.native = get_native_driver(conf.platform, conf)
            except KeyError as ke:  # 配置不全
                self.logger.error(ke)
            except:
                self.logger.warning(f"instantiate {conf.platform} native driver fail")
        self.version = build_version().get("version", "1.2.0")
        self.sdk_version: Version = Version("0.0.0")
        self.dev_tool_version: Version = Version("0.0.0")
        self.platform = "ide"  # ide, android, ios
        # self.logger.info(self.version)
        self.start_cmd = ""
        self._app = None
        self.connection = None
        test_port = (
            str(conf.test_port) if conf.get("test_port", None) else str(TEST_PORT)
        )
        if test_port == "rand":
            test_port = str(pick_unuse_port())
        self.uri = uri + ":" + test_port
        self.test_port = test_port
        self.open_id = conf.get("account_info", {}).get("open_id", None)
        self.ticket = conf.get("account_info", {}).get("ticket", None)
        self.project_path = conf.get("project_path", None)
        self.is_remote = False
        self.is_connected = False
        self.is_audits_running = False  # 体验评分在跑
        self.launch_app_lock = threading.RLock()  # 全局只需有一个launch app的线程
        self.__is_app_relaunching = False
        self.last_launch_cost_time = 1  # 最后一次拉起小程序需要的时间
        self.last_launch_error = None  # 成功为none

        try:
            self.launch_weapp()

            # IdeNative实例有mini属性，因为IdeNative接口通过minium实现
            if self.platform == "ide" and self.native and hasattr(self.native, "mini"):
                self.native.mini = self
            # 根据配置注入一些必要的代码
            if self.app and not self.app.is_injected:
                self._inject()
        except:
            ee.remove_listener("ide_closed", self._relaunch_ide)  # 防止创建失败的minium实例仍引起relaunch
            raise

    @property
    def app(self) -> App:
        return self._app

    @app.setter
    def app(self, v: App or None):
        if v:
            setInjectJsMode(v.js_mode)  # 每次更新app实例时更新一下
        self._app = v

    def __get_dev_tool_info(self):
        try:
            result = self.connection.send("Tool.getInfo", max_timeout=3).result
        except:
            # not support Tool.getInfo
            return
        self.sdk_version = Version(result.SDKVersion)
        if self.sdk_version < Version("2.7.3"):
            raise MiniLaunchError("基础库版本[%s]过低，请确认基础库版本>=2.7.3" % self.sdk_version)
        self.dev_tool_version = Version(result.version)

    def __setup_dev_tool(self):
        if self.conf.get("dev_tool_path", None):  # 配置了cli路径(一定需要是路径), 检查路径是否正确
            if isWindows and not (
                self.conf.dev_tool_path.endswith("cli")
                or self.conf.dev_tool_path.endswith("cli.bat")
            ):
                raise MiniConfigError(
                    "[dev_tool_path] is not correct, it's usually named as 'cli' or 'cli.bat'"
                )
            self.dev_tool_path = self.conf.dev_tool_path
            if not os.path.exists(self.dev_tool_path):
                raise MiniConfigError(
                    "[dev_tool_path] is not correct, it should be a absolute path"
                )
            if isWindows and os.path.exists(self.dev_tool_path):
                add_path_to_env(os.path.dirname(self.dev_tool_path))
                self.dev_tool_path = os.path.basename(self.dev_tool_path)
        else:  # 检查默认配置
            if isMacOS:
                if os.path.exists(MAC_DEVTOOL_PATH):
                    self.dev_tool_path = MAC_DEVTOOL_PATH
                elif self._do_shell("which cli")[0].strip():  # 看看cli是否已经加到path, 可以直接调用
                    self.dev_tool_path = "cli"
                else:
                    self.logger.warning(
                        "default dev_tool_path[%s] not exists and command[cli] not found"
                        % MAC_DEVTOOL_PATH
                    )
                    self.dev_tool_path = None
            elif isWindows:
                if os.path.exists(WINDOWS_DEVTOOL_PATH):
                    # "${WINDOWS_DEVTOOL_PATH}" auto --project xxx --auto-port 9420  ---  有些用户有问题
                    # add path to env, cli.bat auto --project xxx --auto-port 9420  --- 最新使用这种
                    add_path_to_env(os.path.dirname(WINDOWS_DEVTOOL_PATH))
                    self.dev_tool_path = os.path.basename(WINDOWS_DEVTOOL_PATH)
                elif self._do_shell("where cli.bat")[
                    0
                ].strip():  # 看看cli是否已经加到path, 可以直接调用
                    self.dev_tool_path = "cli.bat"
                else:
                    self.logger.warning(
                        "default dev_tool_path[%s] not exists and command[cli.bat] not found"
                        % WINDOWS_DEVTOOL_PATH
                    )
                    self.dev_tool_path = None
            else:
                self.logger.warning("Dev tool doesn't support current OS yet")
                self.dev_tool_path = None

        # 用到project path的时候一定需要cli path
        if self.project_path and not self.dev_tool_path:
            raise MiniConfigError(
                "dev_tool_path not exists and not config default cli command"
            )

    def __del__(self):
        if self.native and self.conf.auto_authorize:
            self.native.release_auto_authorize()
        self.native = None

    def __getattr__(self, name):
        """
        当minium中方法不存在且native中存在，返回native的方法
        """
        if name != "native" and self.native:
            item = getattr(self.native, name, None)
            if callable(item):
                return item
        raise AttributeError(
            "'%s' object has no attribute '%s'" % (self.__class__.__name__, name)
        )

    def __wait_app_initialized(self, timeout, signal):
        """
        等待 {signal} 信号, 代表小程序ready了, 可以响应命令了
        """
        self.__wait_for_initialized_time = time.time()
        self.logger.info(
            "wait for [%s], start at %d" % (signal, self.__wait_for_initialized_time)
        )
        ret = self.connection.wait_for(method=signal, max_timeout=timeout)
        ctime = time.time()
        if not ret and (ctime - self.__wait_for_initialized_time) < timeout:
            # 没有监听到信号，同时等待期间时间有更新
            self.logger.warning(
                "rewait [%s] for %f"
                % (signal, timeout - (ctime - self.__wait_for_initialized_time))
            )
            ret = self.connection.wait_for(
                method=signal,
                max_timeout=timeout - (ctime - self.__wait_for_initialized_time),
            )
        return ret

    def _evaluate_js(
        self,
        filename,
        args=None,
        sync=True,
        default=None,
        code_format_info=None,
        **kwargs,
    ):
        """
        重写, 默认使用self.app.js_mode参数
        """
        return super(WXMinium, self)._evaluate_js(
            filename, args, sync, default, code_format_info, self.app.js_mode, **kwargs
        )

    def _inject(self):
        # conf.mock_native_modal
        use_native = getattr(self.native, "use_native", False)
        if self.platform == "ide" and isinstance(
            self.conf.mock_native_modal, dict
        ):  # IDE上不支持部分原生接口, 用特殊方法mock
            # mini_auth_setting: 记录MOCK过的授权窗，如果确认过授权，后面不需要再通过handle来点击;
            # mock_native_modal_list: 回调栈;
            # handle_mock_native_modal: 弹窗操作;
            # handle_mock_map_modal: 地图操作;
            # mock_native_modal: 添加回调栈;
            # mock_map_modal: 添加地图操作回调栈
            has_mock_native_modal = self._evaluate_js("ideMockModal")
            if not has_mock_native_modal:
                # mock wx.showModal
                not use_native and self._mock_show_modal()
                # mock wx.showActionSheet, action sheet不支持ide的native操作
                self._mock_show_action_sheet()
                # mock wx.requestSubscribeMessage
                not use_native and self._mock_request_subscribe_message()
                # 授权相关接口需要先获取SETTING
                mini_setting = (
                    self.app.call_wx_method("getSetting")
                    .get("result", {})
                    .get("result")
                )
                # 获取到原始setting，mock这个接口
                not use_native and self.app._mock_wx_js("getSetting", "ideMockAuthSetting")
                # 因为不是真正授权了，只能返回一个假的授权状态
                not use_native and self.app._mock_wx_js("authorize", "ideMockAuth")
                # mock wx.getLocation，先看看是否有`scope.userLocation`权限，如果已有，则不需要MOCK
                if (
                    not use_native and mini_setting["authSetting"].get("scope.userLocation") is None
                ):  # 没有授权，mock掉使之不弹窗
                    self._mock_get_location(
                        self.conf.mock_native_modal.get("location", {})
                    )
                # mock wx.chooseLocation, 授权过获取位置也应该mock掉，不然会弹窗
                if not use_native and mini_setting["authSetting"].get("scope.userLocation") in (
                    None,
                    True,
                ):
                    self._mock_choose_location(
                        self.conf.mock_native_modal.get("location", {}),
                        self.conf.mock_native_modal.get("locations", {}),
                    )
                # mock getWeRunData, 没有授权，mock掉使之不弹窗, 返回的参数由于有加密信息，直接mock数据会导致后面后台解密过不去，需要添加配置来输入MOCK数据
                if not use_native and mini_setting["authSetting"].get("scope.werun") is None:
                    self._mock_get_we_run_data(
                        self.conf.mock_native_modal.get("weRunData", {}).get(
                            "encryptedData", "testencryptedData"
                        ),
                        self.conf.mock_native_modal.get("weRunData", {}).get(
                            "iv", "testiv"
                        ),
                    )
                # mock getUserProfile, 基础库2.10.4后才支持
                if self.sdk_version >= Version("2.10.4"):
                    not use_native and self._mock_get_user_profile(
                        self.conf.mock_native_modal.get("userInfo", {})
                    )

        # conf.mock_request
        self._evaluate_js("mockNetwork", code_format_info={"interface": "request"})  # 先mock, 再打开network panel才能记录mock的请求
        if self.conf.mock_request and isinstance(self.conf.mock_request, (list, tuple)):
            for item in self.conf.mock_request:
                try:
                    self.app.mock_request(**item)
                except Exception as e:
                    self.logger.exception(
                        "mock_request config error, configure item is %s, error is %s"
                        % (str(item), str(e))
                    )

        # conf.auto_authorize
        if self.conf.auto_authorize and self.native:  # 需要native实例来处理
            self._set_auto_authorize()

        # conf.mock_images_dir
        # 需要进行mock的图片放置的文件夹
        # conf.mock_images
        # 需要进行mock的图片的kv对
        if self.conf.mock_images_dir or self.conf.mock_images:
            # mockChooseImage
            self.app.reflesh_mocked_images(
                self.conf.mock_images_dir, self.conf.mock_images
            )

    def _set_auto_authorize(self):
        self.native.set_auto_authorize()
        # hook 可能引起弹窗的函数: authorize, getLocation, chooseLocation, getWeRunData, getUserProfile
        def notify(*args):
            self.native.notify()

        self.app.hook_wx_method("authorize", after=notify)
        self.app.hook_wx_method("getLocation", after=notify)
        self.app.hook_wx_method("chooseLocation", after=notify)
        self.app.hook_wx_method("getWeRunData", after=notify)
        self.app.hook_wx_method("getUserProfile", after=notify)
        notify()  # 小程序可能刚起来就会有授权弹窗

    def _mock_show_modal(self):
        """
        mock showModal, 不弹窗, 但可以用native方法调用
        """
        self.app._mock_wx_js("showModal", "ideMockShowModal")

    def _mock_show_action_sheet(self):
        """
        mock showActionSheet, 不弹窗, 但可以用native方法调用
        """
        self.app._mock_wx_js("showActionSheet", "ideMockShowActionSheet")

    def _mock_request_subscribe_message(self):
        """
        mock requestSubscribeMessage, 不弹窗, 但可以用native方法调用
        """
        self.app._mock_wx_js("requestSubscribeMessage", "ideMockSubscribeMessage")

    def _mock_get_location(self, location=None):
        """
        mock getLocation, 不弹窗, 但可以用native方法调用
        """
        if location is None:
            location = {}
        self.app._mock_wx_js("getLocation", "ideMockGetLocation", location)

    def _mock_choose_location(self, location, locations):
        """
        mock chooseLocation, 不弹窗, 但可以用native方法调用
        """
        self.app._mock_wx_js(
            "chooseLocation", "ideMockChooseLocation", (location, locations)
        )

    def _mock_get_we_run_data(self, encrypted_data, iv):
        """
        mock getWeRunData, 不弹窗, 但可以用native方法调用
        """
        self.app._mock_wx_js(
            "getWeRunData", "ideMockGetWeRunData", (encrypted_data, iv)
        )

    def _mock_get_user_profile(self, user_info):
        """
        mock getUserProfile, 不弹窗, 但可以用native方法调用
        """
        self.app._mock_wx_js("getUserProfile", "ideMockGetUserProfile", user_info)

    def _dev_cli(self, cmd, input=b""):
        cmd_template = "%s %s"
        # do_shell 返回 output & error, 但是，实际上cli命令out返回的是error message, err返回的是output message
        err, out = self._do_shell(cmd_template % (self.dev_tool_path, cmd), input=input)
        if err:
            # error like:
            # [error] {
            #   code: 10,
            #   message: 'Error: 错误 Error: Port 9422 is in use (code 10)Error: Port 9422 is in use\n' +
            #     '    at Object.exports.auto [as method] (f6d9ab0.js:2:925)\n' +
            #     '    at processTicksAndRejections (node:internal/process/task_queues:93:5)\n' +
            #     '    at async f8c86db.js:2:3593\n' +
            #     '    at async Object.<anonymous> (df8c86db.js:2:2983)'
            # }
            reg = re.search(
                r"(\[error\]\s*{\s*code:\s*(\d+),\s*message:.*(Error:.*\(code\s\d+\)))",
                err,
                re.M,
            )
            if reg:
                # 提取message信息返回
                return reg.group(3)
            return err
        return

    def _get_system_info(self):
        """
        description: 获取当前系统信息，更新platform/sdk_version
        param {*} self
        return {*}
        """
        try:
            system_info = self.get_system_info()
        except MiniTimeoutCauseByConnectionBreakError:
            # 链接断链了
            raise
        except MiniTimeoutError as te:
            # 命令没响应，应该是基础库版本问题
            if self.sdk_version < Version("2.7.3"):
                raise MiniLaunchError("基础库版本[%s]过低，请确认基础库版本>=2.7.3" % self.sdk_version) from te
            raise
        self.sdk_version = Version(
            system_info.get("SDKVersion", None) or self.sdk_version.version
        )
        platform = system_info.get("platform", "ide").lower()
        self.platform = "ide" if platform == "devtools" else platform

    def launch_weapp(self):
        """
        拉起小程序
        """
        self.__setup_dev_tool()
        stime = time.time()
        self.launch_dev_tool()
        if self.conf.get("platform", None) != "ide" and self.platform  == "ide":
            # launch/connect后仍然是ide, 需要启动远程调试
            try:
                path = self.enable_remote_debug(
                    use_push=self.conf.get("use_push", True),
                    connect_timeout=self.conf.get("remote_connect_timeout", 180),
                )
            except RemoteDebugConnectionLost:  # 远程连接建立后仍然报 remote lost, 可能是小程序挂了
                self.release()
                raise
            except MiniLaunchError as ml:  # 怎么都拉不起, 尝试重启小程序
                if self.native:
                    self.native.start_wechat()
                raise ml
            self.qr_code = path
        self.last_launch_cost_time = time.time() - stime

    @property
    def is_app_relaunching(self):
        return self.__is_app_relaunching

    def _relaunch_ide(self, *args):
        """重新拉起ide
        """
        if not self.app:
            self.logger.error("relaunch app but app is never launch successfully, pass")
            return
        def _relaunch():
            if not self.launch_app_lock.acquire(False):
                self.logger.info("other thread relaunching app")
                # 已经有线程在 launch weapp
                return
            self.logger.info("start relaunch app %d" % threading.get_ident())
            self.__is_app_relaunching = True
            stime = time.time()
            try:
                if self.connection:
                    self.connection.destroy()
                self.close_project()
                self.launch_dev_tool()
                if self.conf.get("platform", None) != "ide" and self.platform  == "ide":
                    # launch/connect后仍然是ide, 需要启动远程调试
                    try:
                        path = self.enable_remote_debug(
                            use_push=self.conf.get("use_push", True),
                            connect_timeout=self.conf.get("remote_connect_timeout", 180),
                        )
                    except RemoteDebugConnectionLost:
                        self.release()
                        raise
                    except MiniLaunchError as ml:  # 怎么都拉不起, 尝试重启小程序
                        if self.native:
                            self.native.start_wechat()
                        raise ml
                    self.qr_code = path
                self.last_launch_cost_time = time.time() - stime
                self.last_launch_error = None
            except Exception as e:
                self.last_launch_error = e
                self.logger.exception(e)
                if self.connection:
                    self.connection.destroy()
            finally:
                self.logger.info("end relaunch app")
                self.__is_app_relaunching = False
                self.launch_app_lock.release()
        t = threading.Thread(target=_relaunch)
        t.setDaemon(True)
        t.start()

    def __update_project_config(self):
        project_config_path = os.path.join(self.project_path, "project.config.json")
        if os.path.isfile(project_config_path):
            conf = {}
            setting = {}
            with open(project_config_path, "r", encoding="UTF-8") as fd:
                j = json.loads(fd.read().strip())
            self.sdk_version = Version(
                j.get("libVersion", "").strip() or self.sdk_version.version
            )
            if self.sdk_version < Version("2.7.3"):  # 尝试fix基础库版本信息
                conf["libVersion"] = "latest"  # latest应该是不合法的
            # update setting
            if j.get("setting", None) is None:
                j["setting"] = {}
            if (
                self.conf.audits is not None
            ):  # 即使工具启动了，修改config.json依然会开始运行体验评分，所以每次launch认为audit running
                setting["autoAudits"] = self.conf.audits
            if conf:  # 需要更新
                j.update(conf)
                j["setting"].update(setting)
                json.dump(j, open(project_config_path, "w", encoding="UTF-8"), indent=2)

    def close_project(self):
        if not self.start_cmd:
            return
        if self.project_path and "--auto-account" not in self.start_cmd:
            self._dev_cli(f'close --project "{self.project_path}"')

    def launch_dev_tool(self):
        # start dev tool with minium model
        self.logger.info("Starting dev tool and launch MiniProgram project ...")
        is_port_in_use = False
        if self.project_path:
            if not os.path.exists(self.project_path):
                raise MiniConfigError("project_path: %s not exists" % self.project_path)
            if not os.path.isdir(self.project_path):
                raise MiniConfigError(
                    "project_path: %s is not directory" % self.project_path
                )
            if not os.path.isfile(
                os.path.join(self.project_path, "project.config.json")
            ):
                raise MiniConfigError(
                    "can't find project.config.json in %s, please confirm the directory contains"
                    " miniproject project" % self.project_path
                )
            self.__update_project_config()
            # config start cmd
            self.start_cmd = 'auto --project "%s" --auto-port %s' % (
                self.project_path,
                self.test_port,
            )
            if self.open_id:
                self.start_cmd += f" --auto-account {self.open_id}"
            elif self.ticket:
                self.start_cmd += f" --test-ticket {self.ticket}"
            # run cmd
            err_msg = self._dev_cli(self.start_cmd, input=b"y")
            if err_msg:
                # launch error
                if "Port %s is in use" % self.test_port in err_msg:
                    # 开发者工具可能已经打开并占用了测试端口, 可直接尝试连接
                    is_port_in_use = True
                else:
                    # 其他错误直接报错看看什么问题
                    raise MiniLaunchError(err_msg)
            else:
                # launch success, wait ide init
                if isWindows:
                    # windows更卡
                    time.sleep(10)
                else:
                    time.sleep(5)
        else:
            self.start_cmd = None
            self.logger.warning(
                "Can not find `project_path` in config, that means you must open dev tool by"
                " automation way first"
            )
            self.logger.warning(
                "If you are not running command like [cli auto --project /path/to/project"
                " --auto-port 9420], you may config `project_path` or run this command first"
            )
            err_msg = None
        try:
            self.connect_dev_tool()
            return
        except MiniConnectError as e:
            # ws连接不上
            self.logger.error(f"{str(e)}, restart now...")
            if is_port_in_use:
                # 端口被占用，先尝试关掉项目再重试
                if self.connection:
                    self.connection.send("Tool.close")
                elif "--auto-account" not in self.start_cmd:
                    self.close_project()
                else:
                    raise MiniLaunchError(
                        "In the case with multi-account mode and no connection with dev tool,"
                        " please close devtools by your self"
                    ) from e
                time.sleep(5)
            if self.start_cmd:  # 重启一次
                self.logger.info("Starting dev tool again...")
                err_msg = self._dev_cli(self.start_cmd, input=b"y")
                if err_msg:
                    raise MiniLaunchError(err_msg) from e
                else:
                    self.logger.info("Restart success")
                    # launch success, wait ide init
                    if isWindows:
                        # windows更卡
                        time.sleep(10)
                    else:
                        time.sleep(5)
                self.connect_dev_tool()
                return
            raise e
        except MiniLaunchError:
            # 初始化app error
            raise

    def connect_dev_tool(self):
        i = 3
        while i:
            try:
                self.logger.info("Trying to connect Dev tool ...")
                self.connection = Connection.create(
                    self.uri, timeout=self.conf.get("request_timeout")
                )
                self.__get_dev_tool_info()
                self._get_system_info()
                self.app = App(
                    self.connection,
                    self.conf.get("auto_relaunch"),
                    native=self.native,
                    platform=self.platform,
                )
                ee.on("ide_closed", self._relaunch_ide)
            except RemoteDebugConnectionLost:  # 链接上了, 但小程序没有响应
                raise
            except MiniLaunchError:
                raise
            except Exception as e:
                self.logger.exception(e)
                i -= 1
                if i == 0:
                    raise MiniConnectError(
                        "three times try to connect Dev tool has all fail ..."
                    )
                continue
            else:
                break
        self.is_connected = True
        self.is_audits_running = self.conf.audits
        return True

    def launch_dev_tool_with_login(self):
        # login first
        if not self.dev_tool_path or not os.path.exists(self.dev_tool_path):
            raise MiniConfigError("dev_tool_path: %s not exists" % self.dev_tool_path)
        if isWindows:
            cmd_template = '"%s"  login'
        else:
            cmd_template = "%s login"
        # 需要输出二维码，所以直接用os.system就好了
        ret = os.system(cmd_template % self.dev_tool_path)
        if ret == 0:
            return self.launch_dev_tool()

    def get_app_config(self, *names):
        """
        获取 app 的配置
        :return: object
        """
        return self._evaluate_js("getAppConfig", args=names)

    def get_system_info(self):
        """
        获取系统信息
        :return:
        """
        return (
            self._call_wx_method("getSystemInfoSync")
            .get("result", {"result": {}})
            .get("result", None)
        )

    def enable_remote_debug(self, use_push=True, path=None, connect_timeout=180):
        """
        开启真机调试
        监听信号:
        v1: App.initialized
        v2: Tool.onRemoteDebugConnected
        """
        self.reset_remote_debug()
        time.sleep(2)
        RETRY_TIMES = 3
        if use_push:
            retry_times = RETRY_TIMES
            while retry_times > 0:
                thread1 = None
                thread2 = None
                semaphore = threading.Semaphore(0)  # 用Semaphore标记其中一个信号已到达
                try:
                    thread1 = WaitThread(
                        target=self.__wait_app_initialized,
                        args=(
                            connect_timeout + 30 * (RETRY_TIMES - retry_times),
                            "App.initialized",
                        ),
                        semaphore=semaphore,
                    )
                    thread1.setDaemon(True)
                    thread1.start()
                    thread2 = WaitThread(
                        target=self.__wait_app_initialized,
                        args=(
                            connect_timeout + 30 * (RETRY_TIMES - retry_times),
                            "Tool.onRemoteDebugConnected",
                        ),
                        semaphore=semaphore,
                    )
                    thread2.setDaemon(True)
                    thread2.start()
                    self.logger.info(
                        f"Enable remote debug, for the {4 - retry_times}th times"
                    )
                    self.connection.send(
                        "Tool.enableRemoteDebug",
                        params={"auto": True},
                        max_timeout=connect_timeout,
                    )
                    self.is_remote = True
                    # 成功发起远程调试后, update一下等待时间
                    self.__wait_for_initialized_time = time.time()
                except Exception as e:
                    retry_times -= 1
                    self.logger.error("enable remote debug fail ...")
                    self.logger.error(e)
                    if retry_times == 0:
                        self.logger.error(
                            "Enable remote debug has been fail three times. Please check your"
                            " network or proxy effective or not "
                        )
                        raise
                    continue
                else:
                    retry_times -= 1
                    ret1 = ret2 = False
                    semaphore.acquire()  # 等待初始化信号
                    if thread1:
                        ret1 = thread1.get_result(block=False) or False
                    if thread2:
                        ret2 = thread2.get_result(block=False) or False
                    if retry_times == 0 and ret1 is False and ret2 is False:
                        self.logger.error(
                            "Wait for APP initialized has been fail three times. Please check your"
                            " phone's current foreground APP is WeChat or not, and check"
                            " miniProgram has been open or not "
                        )
                        raise MiniLaunchError("Launch APP Error")
                    if ret1 is True or ret2 is True:
                        break
                    else:
                        # 等不到App.initialized一般就是白屏，通道不通，exit会报错，跳出重试，需要catch
                        try:
                            self.app.exit()
                        except (MiniTimeoutError, MiniConnectionClosedError):
                            # 白屏的时候切换一下扫码编译，有时会有意想不到效果
                            self.connection.send(
                                "Tool.enableRemoteDebug",
                                params={"auto": False},
                                max_timeout=connect_timeout,
                            )
            # 远程调试开启成功后，小程序运行态已经转到手机，需要重新实例化app
            self._get_system_info()
            self.app = App(
                self.connection,
                self.conf.get("auto_relaunch"),
                native=self.native,
                platform=self.platform,
            )
            setattr(self.connection, "_is_close_app_by_cmd", False)  # 重新拉起重置一下
            if not self.app.is_injected:
                self._inject()
            return None
        if path is None:
            path = os.path.join(self.conf.outputs, "debug_qrcode.jpg")
        qr_data = self.connection.send(
            "Tool.enableRemoteDebug", max_timeout=connect_timeout
        ).result.qrCode
        with open(path, "wb") as qr_img:
            qr_img.write(base64.b64decode(qr_data))

        return path

    @catch
    def reset_remote_debug(self):
        """
        重置远程调试，解决真机调试二维码扫码界面报-50003 等常见错误
        :return:
        """
        return self.connection.send("Tool.resetRemoteDebug")

    def clear_auth(self):
        """
        清除用户授权信息. 公共库 2.9.4 开始生效
        :return:
        """
        if (
            self.conf.mock_native_modal and self.conf.platform == "ide"
        ):  # 对授权弹窗MOCK了，需要清除MOCK信息
            self._evaluate_js("clearMockAuth")
        self.connection.send("Tool.clearAuth")

    def get_test_accounts(self):
        """
        获取已登录的真机账号
        :return: list [{openid, nickName}]
        """
        return self.connection.send("Tool.getTestAccounts").result.accounts

    def shutdown(self):
        """
        关闭 Driver, 释放native引用
        :return: status
        """
        ee.remove_listener("ide_closed", self._relaunch_ide)
        if self.native and getattr(self.native, "mini", None):
            self.native.mini = None  # IDE native借用minium通信通道交互
        self.native = None
        # 释放app
        if self.is_remote and self.app:
            try:
                self.logger.info("MiniProgram closing")
                self.app.exit()
            except (
                MiniTimeoutError,
                MiniAppError,
                WebSocketConnectionClosedException,
                MiniConnectionClosedError,
            ) as e:
                self.logger.exception(f"Close app excrption:{e}")
            finally:
                self.is_remote = False
                if self.app is not None:
                    self.app.native = None
                    self.app = None
        # 释放connection
        if self.connection:
            try:
                setattr(self.connection, "_is_close_by_cmd", True)
                self.connection.send_async("Tool.close", ignore_response=True)
            except (WebSocketConnectionClosedException, MiniConnectionClosedError, MiniClientOfflineError):
                pass
            finally:
                if self._wait(
                    lambda: not self.connection._is_connected, 5, 1
                ):  # already closed
                    self.connection.remove_all_observers()
                else:
                    self.connection.destroy()
                self.connection = None

    def stop_audits(self, report_path=None):
        """
        停止体验评分
        """
        if self.is_audits_running and self.app.platform == "ide":  # 仅工具上有意义
            self.logger.info("stopping audits")
            self.is_audits_running = False
            try:
                self.app.stop_audits(
                    report_path
                    or (
                        self.conf.outputs
                        and os.path.join(self.conf.outputs, "audits.html")
                    )
                )
            except (
                MiniTimeoutError,
                MiniAppError,
                WebSocketConnectionClosedException,
                MiniConnectionClosedError,
            ) as e:
                self.logger.exception(f"Stop audits excrption:{e}")

    def release(self):
        """
        释放所有资源, 不允许在case中调用
        :return: None
        """
        native = self.native
        # 停止体验评分, 结果存outputs目录
        self.stop_audits()
        self.shutdown()
        # 清理一下手机弹窗
        if self.conf.platform != "ide" and native:
            native.close_local_debug_modal()
        native = None

    def wait_app_relaunch(self, timeout=None) -> None or Exception:
        wait_launch_time = self.last_launch_cost_time + 20 if timeout is None else timeout
        self.logger.warning(f"app is relaunching, wait {wait_launch_time} seconds for it")
        if not self.launch_app_lock.acquire(timeout=wait_launch_time):
            return TimeoutError(f"app did not relaunch within {wait_launch_time} seconds")
        self.logger.warning("app is relaunch complete")
        self.launch_app_lock.release()
        if self.last_launch_error:
            return self.last_launch_error


if __name__ == "__main__":

    mini = WXMinium()



#!/usr/bin/env python3
# Created by yopofeng on 2021-11-22
# define exceptions for native
from enum import Enum


class ResetError(Enum):
    OK = 0
    ERROR = -1
    RELAUNCH_MINIPROGRAM = -2
    RELAUNCH_APP = -3


class ModalStatus(Enum):
    OK = 0
    Error = -1
    NotOfficialModal = -2  # 不是官方提供的弹窗
    NotFound = -3



#!/usr/bin/env python3
# Created by xiazeng on 2019-05-31
from lib2to3.pgen2.token import SLASH
import sys
import os.path
import argparse
import platform

# import unittest
import logging.handlers
import json
import glob
import fnmatch
import copy
import time
import websocket
import threading

from minium.framework.minisuite import MiniSuite
from minium.framework.session import Session
from multiprocessing import Queue
from minium import build_version
from minium.framework import caseinspect
from minium.framework import minitest
from minium.framework import miniconfig
from minium.framework import miniresult
from minium.miniprogram.wx_minium import WXMinium
from minium.framework.exception import *
from minium.framework.libs import unittest
from minium.framework.wording import WORDING

unittest.TestLoader.suiteClass = unittest.TestSuite

logger = logging.getLogger("minium")
g_case_list = []
FILENAME_SUMMARY = "summary.json"
is_connected = False
test_response = None


def update_summary(summary_path: str, summary: dict):
    if os.path.isfile(summary_path):
        with open(summary_path, "r", encoding="utf8") as fd:
            _summary = json.loads(fd.read() or "{}")
    else:
        _summary = {}
    _summary.update(summary)
    with open(summary_path, "w", encoding="utf8") as fd:
        json.dump(_summary, fd, indent=4)


def run_tests(tests):
    result = miniresult.MiniResult()
    tests.run(result)
    result.print_shot_msg()
    return result


def load_from_suite(case_path, json_path):
    """
    从json文件或者 json 字符串中获取用例筛选规则，并加载用例
    :param case_path: 用例存放的路径
    :param json_path:
    :return tests: test cases
    :return mini_suite: MiniSuite
    """
    mini_suite = MiniSuite(json_path)
    test_list = unittest.TestSuite()
    logger.debug("==========Scan Cases from suite===========")
    if mini_suite.case_list:  # 新版的读这个字段，按顺序loadcase
        for (
            item
        ) in (
            mini_suite.case_list
        ):  # item: {"pkg": "package1", "case_name": "casename1", "class_name": "classname1"}
            pkg = item["pkg"]
            case_name = item["case_name"]
            # import all module in pkg
            module_case_info_list = caseinspect.load_module(
                case_path, pkg, mini_suite=mini_suite
            )
            for module_name, module_info in module_case_info_list.items():
                logger.debug(f"- {module_name}")
                for case_info in module_info["case_list"]:
                    if fnmatch.fnmatch(case_info["name"], case_name):
                        logger.debug(
                            f"  |--{module_info['class_name']}.{case_info['name']}"
                        )
                        case = load_from_case_name(module_name, case_info["name"])
                        test_list.addTests(case)
        g_case_list.append(test_list)
    else:
        for (
            pkg_cases
        ) in (
            mini_suite.pkg_list
        ):  # pkg_cases: {"pkg": "packagename", "case_list": [*casenames]}
            tests = unittest.TestSuite()
            pkg = pkg_cases["pkg"]
            case_list = pkg_cases["case_list"]
            for case_suite_info in case_list:
                if isinstance(case_suite_info, str):
                    case_name = case_suite_info
                else:
                    case_name = case_suite_info["name"]
                module_case_info_list = caseinspect.load_module(
                    case_path, pkg, mini_suite=mini_suite
                )
                for module_name, module_info in module_case_info_list.items():
                    logger.debug(f"- {module_name}")
                    for case_info in module_info["case_list"]:
                        if fnmatch.fnmatch(case_info["name"], case_name):
                            logger.debug(
                                f"  |--{module_info['class_name']}.{case_info['name']}"
                            )
                            case = load_from_case_name(module_name, case_info["name"])
                            tests.addTests(case)
            g_case_list.append(tests)
            test_list.addTests(tests)
    return test_list, mini_suite


def load_from_module(module):
    """
    通过包名加载该 package 下所有用例
    :param module:
    :return:
    """
    logger.debug(f"Loading module: {module}")
    loader = unittest.TestLoader()
    test_module = caseinspect.import_module(module)
    tests = loader.loadTestsFromModule(test_module)
    logger.debug(f"put module[{module}] into queue")
    g_case_list.append(tests)
    return tests


def load_single_case(module, case_name):
    """
    加载单条 case
    :param module:
    :param case_name:
    :return:
    """
    tests = load_from_case_name(module, case_name)
    g_case_list.append(tests)
    return tests


def load_from_case_name(module, case_name):
    """
    通过包名和用例名加载用例
    :param module:
    :param case_name:
    :return:
    """
    logger.debug(f"Loading Case: {module}.{case_name}")
    loader = unittest.TestLoader()
    test_class = caseinspect.find_test_class(module, case_name)
    if test_class:
        tests = loader.loadTestsFromName(case_name, test_class)
        logger.debug(f"put case[{case_name}] into queue")
        return tests
    else:
        raise AssertionError(
            "can't not find testcase %s in module %s" % (case_name, module)
        )


def test_ws_connection(test_port):
    def on_open(ws):
        global is_connected
        print("connection opened")
        is_connected = True

    def on_error(ws, error):
        print("error: %s" % str(error))
        exit(1)

    def on_message(ws, message):
        global test_response
        print("receive: %s" % message)
        message = json.loads(message)
        if message["id"] == "minitest-test-id":
            test_response = message

    def on_close(ws):
        global is_connected
        print("closed")
        is_connected = False

    url = "ws://localhost:{}".format(test_port)
    print("start connect {}".format(url))
    client = websocket.WebSocketApp(
        url,
        on_open=on_open,
        on_message=on_message,
        on_error=on_error,
        on_close=on_close,
    )

    def run_forever():
        client.run_forever()

    thread = threading.Thread(target=run_forever, args=())
    thread.daemon = True
    thread.start()
    s = time.time()
    while time.time() - s < 10 and thread.is_alive():
        if is_connected:
            print("connect to WebChatTools successfully")
            print("连接开发者工具成功")
            break
        time.sleep(1)
    else:
        print("connect WebChatTools fail, please confirm the test port is open")
        print("连接开发者工具失败，可能的原因有:")
        print("1. 没有打开自动化测试端口，请检查开发者工具的通知栏是否有: 小程序的自动化端口已开启，端口号是 xxxx")
        print("2. 调试基础库版本过低，请选择最新版本")
        exit(1)
    message = json.dumps(
        {
            "id": "minitest-test-id",
            "method": "App.callWxMethod",
            "params": {"method": "getSystemInfoSync", "args": []},
        },
        separators=(",", ":"),
    )
    client.send(message)
    s = time.time()
    while time.time() - s < 30:
        if test_response:
            print("response successfully")
            print("收到命令返回")
            break
        time.sleep(1)
    else:
        print(
            "can't receive response, please confirm miniprogram is launch successfully"
        )
        print("未收到命令返回，请确认小程序项目是否正确运行(没有白屏/报错)")
        exit(1)
    client.close()


def get_config(config):
    conf_list = []
    if config:
        conf_list = miniconfig.MiniConfig.from_file(config)
        conf_list = [conf_list] if not isinstance(conf_list, list) else conf_list
    else:
        conf_list.append(miniconfig.MiniConfig())
    return conf_list


def run_session(conf_list, run_mode, only_native, generate_report, task_limit_time):
    session_list = list()
    q = Queue()
    for tests in g_case_list:
        q.put(tests)
    for c in conf_list:
        new_q = Queue()
        if run_mode == "fork":
            for tests in copy.deepcopy(g_case_list):
                new_q.put(tests)
            q = new_q
        if only_native:
            c.only_native = True
        logger.info(f"start session with {run_mode} mode \n{c}\n")
        s = Session(conf=c, queue=q, generate_report=generate_report)
        s.start()
        session_list.append(s)

    rest_time = task_limit_time  # 任务剩余时间
    for se in session_list:
        stime = time.time()
        se.join(rest_time)
        if not task_limit_time:
            # 不限制时间
            continue
        # 计算剩余时间
        rest_time = rest_time - (time.time() - stime)
        if rest_time <= 0:  # 超时
            break
    for se in session_list:
        if se.is_alive():
            se.kill()


def check_env(config_list):
    """
    check system environment for minitest
    :param config:
    :return:
    """
    # default dev_tool_path
    MAC_DEVTOOL_PATH = "/Applications/wechatwebdevtools.app/Contents/MacOS/cli"
    WINDOWS_DEVTOOL_PATH = "C:/Program Files (x86)/Tencent/微信web开发者工具/cli.bat"

    if sys.version_info.major < 3 and sys.version_info.minor < 8:
        print("python version is less than 3.8, please update")

    is_mac = True
    SLASH = "/"

    if platform.system().lower() == "windows":
        is_mac = False
        SLASH = "\\"

    for config in config_list:
        dev_tool_path = None
        if config.dev_tool_path:
            dev_tool_path = config.dev_tool_path
        elif is_mac:
            dev_tool_path = MAC_DEVTOOL_PATH
        else:
            dev_tool_path = WINDOWS_DEVTOOL_PATH
        if not os.path.exists(dev_tool_path):
            logger.error("`dev_tool_path` %s not exists, please check", dev_tool_path)
            exit(0)

        project_config = None
        if config.project_path:
            logger.debug("checking project_path")
            if not os.path.exists(config.project_path):
                logger.error(
                    "`project_path` %s in config is not exists", config.project_path
                )
                exit(0)

            logger.debug("checking project.config.json")
            if not os.path.exists(config.project_path + SLASH + "project.config.json"):
                logger.error("project.config.json not found in %s", config.project_path)
                exit(0)
            else:
                with open(
                    config.project_path + SLASH + "project.config.json",
                    "r",
                    encoding="utf8",
                ) as f:
                    project_config = json.load(f)

            if project_config is not None:
                if "miniprogramRoot" in project_config:
                    miniprogramRoot = (
                        config.project_path + SLASH + project_config["miniprogramRoot"]
                    )
                else:
                    miniprogramRoot = config.project_path
                if not os.path.exists(miniprogramRoot):
                    logger.error("miniprogramRoot in project.config.json not exists")
                    exit(0)
                # check app.json
                if not os.path.exists(
                    os.path.abspath(miniprogramRoot) + SLASH + "app.json"
                ):
                    logger.error(
                        f"app.json not exists in `{miniprogramRoot}`, \
please make sure `{miniprogramRoot}` is miniprogram project path"
                    )
                    exit(0)
                if "cloudfunctionRoot" in project_config:  # use tencent cloud
                    cloudfunctionRoot = (
                        config.project_path
                        + SLASH
                        + project_config["cloudfunctionRoot"]
                    )
                    if not os.path.exists(cloudfunctionRoot):
                        logger.error(
                            "cloudfunctionRoot in project.config.json not exists"
                        )
                        exit(0)

            start_cli = os.popen(
                dev_tool_path
                if is_mac
                else ('"%s"' % dev_tool_path)
                + " auto --project "
                + config.project_path
                + "--auto-port 9420"
            )
            print(start_cli)


def main():
    # bug: 在 fork 模式下Python 会出现 crash
    # crashed on child side of fork pre-exec
    # Invalid dylib load. Clients should not load the unversioned libcrypto dylib as it does not have a stable ABI.
    import multiprocessing

    multiprocessing.set_start_method("spawn")
    # 解析参数
    parsers = argparse.ArgumentParser()
    parsers.add_argument(
        "-v", "--version", dest="version", action="store_true", default=False, help=WORDING.HELP.version
    )
    parsers.add_argument(
        "-p",
        "--path",
        dest="path",
        type=str,
        help=WORDING.HELP.path,
        default=None,
    )
    parsers.add_argument(
        "-m",
        "--module",
        dest="module_path",
        type=str,
        help=WORDING.HELP.module,
    )
    parsers.add_argument(
        "--case",
        dest="case_name",
        type=str,
        default=None,
        help=WORDING.HELP.case_name,
    )
    parsers.add_argument(
        "--module_search_path",
        dest="sys_path_list",
        nargs="*",
        help=WORDING.HELP.module_search_path,
    )
    parsers.add_argument(
        "--apk",
        dest="apk",
        action="store_true",
        default=False,
        help=WORDING.HELP.apk,
    )
    parsers.add_argument(
        "-s",
        "--suite",
        dest="suite_path",
        type=str,
        default=None,
        help=WORDING.HELP.suite,
    )
    parsers.add_argument(
        "-c", "--config", dest="config", type=str, default=None, help=WORDING.HELP.config
    )
    parsers.add_argument(
        "-g",
        "--generate",
        dest="generate",
        action="store_true",
        default=False,
        help=WORDING.HELP.generate
    )

    parsers.add_argument(
        "--mode",
        dest="run_mode",
        type=str,
        default="fork",
        help=WORDING.HELP.mode
    )

    parsers.add_argument(
        "-a",
        "--accounts",
        dest="accounts",
        action="store_true",
        help=WORDING.HELP.accounts
    )

    parsers.add_argument(
        "--only-native",
        dest="only_native",
        action="store_true",
        help=WORDING.HELP.only_native,
    )

    parsers.add_argument(
        "--test-connection",
        dest="test_connection",
        action="store_true",
        help=WORDING.HELP.test_connection
    )

    parsers.add_argument(
        "--test-port",
        dest="test_port",
        type=str,
        default="9420",
        help=WORDING.HELP.test_port
    )

    parsers.add_argument(
        "--task-limit-time",
        dest="task_limit_time",
        type=str,
        default="0",
        help=WORDING.HELP.task_limit_time
    )

    parsers.add_argument(
        "--test",
        dest="just_test",
        action="store_true",
        default=False,
        help=WORDING.HELP.just_test
    )

    parsers.add_argument(
        "--check-env",
        dest="check_env",
        action="store_true",
        default=False,
        help=WORDING.HELP.check_env
    )

    parsers.format_help()

    parser_args = parsers.parse_args()
    version = parser_args.version
    path = parser_args.path
    module = parser_args.module_path
    apk = parser_args.apk
    case_name = parser_args.case_name
    generate_report = parser_args.generate
    suite_path = parser_args.suite_path
    config = parser_args.config
    sys_path_list = parser_args.sys_path_list
    show_accounts = parser_args.accounts
    run_mode = parser_args.run_mode
    only_native = parser_args.only_native
    test_connection = parser_args.test_connection
    test_port = parser_args.test_port
    task_limit_time = int(parser_args.task_limit_time) or None
    just_test = parser_args.just_test
    is_check_env = parser_args.check_env

    if show_accounts:
        # 打印已登录的多账号
        if config:
            print(WXMinium(get_config(config)[0]).get_test_accounts())
        else:
            # 没有配置的先看看端口是否打开了
            test_ws_connection(9420)
            print(WXMinium().get_test_accounts())
        exit(0)

    if test_connection:
        # 测试ws链接是否正常
        test_ws_connection(test_port)
        exit(0)

    if sys_path_list:
        # 添加 package 寻找路径到 sys.path
        for sys_path in sys_path_list:
            logger.info("insert %s to sys.path", sys_path)
            sys.path.insert(0, sys_path)

    if version:
        # 打印 minium 版本信息
        print(build_version())
        exit(0)

    if apk:
        # 打印 android uiautomator 相关的 apk 包路径
        bin_root = os.path.join(
            os.path.dirname(os.path.dirname(__file__)),
            "native",
            "lib",
            "at",
            "bin",
            "*apk",
        )
        print("please install apk:")
        for filename in glob.glob(bin_root):
            print(f"adb install -r {filename}")
        exit(0)

    if path is None:
        # 用例目录路径
        logger.debug("case directory path not specified, use current path")
        path = os.getcwd()

    if not os.path.exists(path) or not os.path.isdir(path):
        logger.error("case directory: %s not exists" % path)
        parsers.print_help()
        exit(0)

    sys.path.insert(0, path)

    # 提前预加载默认配置，否则处理 suite 的时候logger 参数不生效
    minitest.AssertBase.setUpConfig(default=True)

    # 处理 suite、case、module 参数
    suite = None
    if suite_path:
        tests, suite = load_from_suite(path, suite_path)
        print(f"load {tests.countTestCases()} cases")
        print(
            f'import package {",".join(suite.success_pkg)} success, import package {",".join(suite.fail_pkg)} fail'
        )
    elif module is None:
        # raise RuntimeError("suite_path or pkg is necessary")
        logger.error("suite_path or module_path is necessary at least one")
        parsers.print_help()
        exit(0)
    elif case_name is None:
        load_from_module(module)
    else:
        load_single_case(module, case_name)
    print("after load %s" % suite_path or module)
    for c in g_case_list:
        print(str(c))
    # 处理 config 参数
    if config and not os.path.exists(config):
        # raise RuntimeError("config not exists:%s" % config)
        logger.error("config not exists:%s" % config)
        parsers.print_help()
        exit(0)
    conf_list = get_config(config)
    # 处理一下import error的问题
    if suite and suite.fail_pkg:
        for conf in conf_list:
            if conf.outputs:
                summary_path = os.path.join(conf.outputs, FILENAME_SUMMARY)
                update_summary(summary_path, {"import_error": suite.fail_pkg})

    if is_check_env:
        check_env(conf_list)
        print("check env done")
        exit(0)

    # 输出版本信息
    print(build_version())
    if just_test:
        exit(0)
    # 启动 session
    run_session(conf_list, run_mode, only_native, generate_report, task_limit_time)
    exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# encoding: utf-8
"""
@author: brill
@file: qandroidnative.py
@time: 2019/7/22 10:52 AM
@desc:
"""
import time

import at

from minium.framework.exception import *
from minium.native.qq_native.qbasenative import QBaseNative, NativeError
from ..lib.android_base import UiDefine

QQ_PACKAGE = "com.tencent.mobileqq"
QQ_ACTIVITY = ".activity.SplashActivity"

QQ_MINI_ACTIVITY = "com.tencent.mobileqq.mini.appbrand.ui.AppBrandUI"


class QUiDefine(UiDefine):
    pass


class QAndroidNative(QBaseNative):
    def __init__(self, json_conf):
        super(QAndroidNative, self).__init__(json_conf)
        if json_conf is None:
            json_conf = {}
        self.serial = json_conf.get("serial")
        self.at = at.At(self.serial)
        self.ui = QUiDefine(self.at)

    # 打开scheme
    def open_remote_debug(self, scheme_url):
        self.at.apkapi.launch()
        self.start_qq()
        self.open_scheme(scheme_url)
        self.at.e.cls_name("com.tencent.tbs.core.webkit.WebView").exists(20)

    #  打开qq
    def start_qq(self):
        self.at.adb.start_app(QQ_PACKAGE, QQ_ACTIVITY)

    def start_app(self):
        self.start_qq()

    def stop_app(self):
        self.stop_qq()

    # 关闭QQ
    def stop_qq(self):
        self.at.adb.stop_app(QQ_PACKAGE)

    def open_scheme(self, scheme=None):
        if scheme is None:
            raise NativeError("scheme 不能为空")
        # shell = "am start -a \"android.intent.action.VIEW\" -d \"%s\"" % scheme
        shell = 'am start -d "mqqapi://microapp/open?fakeUrl=%s"' % scheme
        print(shell)
        res = self.at.adb.run_shell(shell)
        print(res)
        return self.weapp_has_launched()

    def screen_shot(self, filename, quality=30):
        return self.at.device.screen_shot(filename, quality=quality)

    def ensure_in_foreground(self):
        act_name = self.at.adb.get_current_activity()
        return act_name

    def weapp_has_launched(self, timeout=30, raise_exception=False):
        time0 = time.time()
        while time.time() - time0 < timeout:
            act_name = self.ensure_in_foreground()
            if act_name.__contains__(QQ_MINI_ACTIVITY):
                return True
            time.sleep(1)
        raise MiniLaunchError("launch mini app timeout")

    def pick_media_file(
        self,
        cap_type="camera",
        media_type="photo",
        original=False,
        duration=5.0,
        index_list=None,
    ):
        pass

    def input_text(self, text):
        pass

    def _allow_authorize(self, answer):
        if answer:
            self.ui.btn_authorize_ok.click()
        else:
            self.ui.btn_authorize_cancel.click()

    def allow_login(self, answer=True):
        self._allow_authorize(answer)

    def allow_get_user_info(self, answer=True):
        self._allow_authorize(answer)

    def allow_get_location(self, answer=True):
        pass

    def handle_modal(self, btn_text="确定", title=None):
        if title:
            self.e.text(title).parent().cls_name(
                "android.widget.FrameLayout"
            ).child().cls_name("android.widget.Button").text(btn_text).click()
        self.e.cls_name("android.widget.Button").text(btn_text).click()

    def handle_action_sheet(self, item):
        self.e.cls_name("android.widget.TextView").text(item).click()

    def forward_miniprogram(self, name, text=None, create_new_chat=True):
        self.ui.action_menu.click()
        self.e.text("转发").click()
        return self.forward_miniprogram_inside(name, text, create_new_chat)

    def forward_miniprogram_inside(self, name, text=None, create_new_chat=True):
        if create_new_chat:
            self.e.text("创建新聊天").click()
            self.e.text_contains(name).click(True)
            self.e.text("确定(1)").enabled(True).click()
        else:
            self.e.text_contains(name).click(True)
        if text:
            self.e.edit_text().enter(text)
        self.e.cls_name("android.widget.Button").text("发送").click()
        self.e.text("已转发").wait_disappear()

    def send_custom_message(self, message=None):
        pass

    def call_phone(self):
        pass

    def handle_picker(self, *items):
        instance = 0
        for item in items:
            input_elem = self.ui.comp_picker_input.instance(instance)
            next_elem = input_elem.parent().child("android.widget.Button")
            first_text = input_elem.get_text()
            while True:
                # todo: 要判断上滑还是下滑
                current_text = input_elem.get_text()
                if current_text == str(item):
                    break
                if first_text == str(item):
                    raise NativeError(f" not found")
            instance += 1

    def get_authorize_settings(self):
        """
        在小程序的授权页面，获取小程序的授权设置
        :return:
        """
        ui_views = self.at.java_driver.dump_ui()
        setting_map = {}
        for ui_view in ui_views:
            if ui_view.cls_name == "android.view.View" and ui_view.content_desc in [
                "已开启",
                "已关闭",
            ]:
                check_status = True if ui_view.content_desc == "已开启" else False
                parant_view = ui_view.sibling().get_children()[0]
                setting_map[parant_view.text] = check_status
        return setting_map

    def back_from_authorize_setting(self):
        """
        从小程序授权页面跳转回小程序
        :return:
        """
        self.at.adb.press_back()

    def authorize_page_checkbox_enable(self, name, enable):
        """
        在小程序授权设置页面操作CheckBox
        :param name: 设置的名称
        :param enable: 是否打开
        :return:
        """
        setting_map = self.get_authorize_settings()
        if setting_map.get(name) == enable:
            return
        self.e.text(name).parent().instance(2).child().cls_name(
            "android.view.View"
        ).click()
        if not enable:
            self.e.cls_name("android.widget.Button").text("关闭授权").click_if_exists(5)

    def release(self):
        self.stop_qq()
        self.at.release()

    @property
    def e(self):
        return self.at.e


if __name__ == "__main__":
    qqnative = QAndroidNative(None)
    qqnative.open_scheme("123123")



# encoding:utf-8
import os.path
import json
import logging
import at.core.uidevice
import at.core.element
import at.core.javadriver
import at.core.config
import at.core.accesshelper
import at.eventmonitor
import at.apkapi
import at.core.adbwrap
import at.jlogcat
import at.core.stf
import at.core.stf.minitouch

uiautomator_version = at.core.config.UIAUTOMATOR2


def build_info():
    """打包的版本信息"""
    config_path = os.path.join(os.path.dirname(__file__), "build_info.json")
    if not os.path.exists(config_path):
        return {}
    else:
        try:
            return json.load(open(config_path, "r", encoding="utf8"))
        except:
            return {}


class At(object):
    """主入口"""
    at_cache = {}
    device_status = {}

    def __init__(self, serial=None, display_id=None, event_path=None, open_atstub=True):
        if serial is None:
            serial = at.core.adbwrap.AdbWrap.get_default_serial()

        self.serial = serial
        self.adb = at.core.adbwrap.AdbWrap.apply_adb(serial)
        self.apkapi = at.apkapi.AppApi(self.adb)
        self.java_driver = at.core.javadriver.JavaDriver.apply_driver(serial, uiautomator_version, open_atstub)
        at.core.element.Element.bind_java_driver(self.java_driver)
        self._logcat = None
        self.device = at.core.uidevice.PyUiDevice(self.java_driver)
        self.access_helper = at.core.accesshelper.AccessHelper(self.java_driver)
        self.event_monitor = at.eventmonitor.EventMonitor(self.java_driver)

        if serial not in At.at_cache:
            At.at_cache[serial] = self
            logging.info("at build info: %s", build_info())

        if event_path:
            minitouch = at.core.stf.minitouch.MiniTouch(self.serial, event_path, display_id)
            self.device.set_input_interceptor(minitouch)
            self.input_interceptor = minitouch
        else:
            self.input_interceptor = None

    @property
    def logcat(self):
        if not self._logcat:
            self._logcat = at.jlogcat.JLogCat(self.adb)
            self._logcat.setDaemon(True)
            self._logcat.start()
        return self._logcat

    @classmethod
    def set_uiautomator_version(cls, version):
        logger.info("set uiautomator_version=%s", version)
        cls.uiautomator_version = version

    def register_hook(self, hook):
        self.java_driver.hook_list.clear()
        self.java_driver.register(hook)

    @property
    def e(self):
        elem = at.core.element.Element(jd_instance=self.java_driver)
        if self.input_interceptor:
            elem.use_custom()
            elem.set_input_interceptor(self.input_interceptor)
        return elem

    def stop_logcat(self):
        if self._logcat:
            self._logcat.stop()

    def release(self):
        at.core.javadriver.JavaDriver.release_driver(self.serial)
        self.stop_logcat()

    def click_image(self, image_path):
        """点击图片，底层用的是`airtest
<https://airtest.readthedocs.io/zh_CN/latest/>`_."""
        import at.vision.template_match
        temp_path = "temp.png"
        source_path = self.device.screen_shot(temp_path)
        r = at.vision.template_match.search(image_path, source_path)
        if not r:
            raise RuntimeError("search %s failed" % image_path)
        else:
            self.device.click_on_point(r['results'][0], r['results'][1])

    def on_at_event(self, event, callback):
        """
        由AtServer返回的事件回调，目前只有click事件
        :param event:
        :param callback:
        """
        self.java_driver.websocket.add_callback(event, callback)


if __name__ == '__main__':
    import logging
    import logging.config
    logging.basicConfig(level=logging.DEBUG)
    logger = logging.getLogger()
    uiautomator_version = at.core.config.UIAUTOMATOR2
    a = At()

    try:
        a.apkapi.add_gallery('/Users/mmtest/Downloads/222.png')
        # print a.e.text(u"消息免打扰").parent().instance(2).child().rid("com.tencent.mm:id/k2").get_desc()
    finally:
        a.release()



#!/usr/bin/env python
# encoding:utf-8
import logging
import logging.config
import unittest

import at
import at.core.config

logging.basicConfig(level=logging.DEBUG,
                    format="%(levelname)-5.5s %(asctime)s %(filename)s %(funcName)s %(lineno)-3d %(message)s")

logger = logging.getLogger()


class AtTestBase(unittest.TestCase):
    def setUp(self):
        logging.basicConfig(level=logging.DEBUG)
        self.at = at.At()
        self.adb = self.at.adb



# encoding:utf-8
# created 2018/11/8
import os.path
import logging
import base64
import requests
import time
import at.utils.commonhelper as commonhelper
from .core import adbwrap

logger = logging.getLogger()


class AppApi(object):
    PACKAGE = "com.tencent.mm.atstub"
    ACT = ".ui.LauncherUI"
    action = "com.tencent.mm.atstub.api"

    def __init__(self, _adb):
        """
        :type adb: adbwrap.AdbWrap
        """
        self.adb = _adb

    def launch(self):
        if not self.adb.pkg_has_installed(self.PACKAGE):
            apk_path = os.path.join(os.path.dirname(__file__), "bin", "AtServer.apk")
            if not self.adb.install(apk_path):
                logger.error("%s install failed", apk_path)
                return False
        r = self.adb.start_app(self.PACKAGE, self.ACT)
        time.sleep(2)
        return r

    def add_gallery(self, path):
        filename = os.path.basename(path)
        with open(path, "rb") as f:
            encode_byte = base64.b64encode(f.read())
            encode_str = encode_byte.decode()

        port = commonhelper.pick_unuse_port()
        self.adb.forward(port, 56788)
        session = requests.Session()
        session.trust_env = False
        r = session.request("post", "http://localhost:%d/storage" % port,
                            params={"filename": filename, "q": "postBase64Image", "data": encode_str})
        self.adb.forward_remove(port)
        logger.debug(r.text)
        r.raise_for_status()

    def scan_gallery(self, name):
        """
        Android 8将会不兼容，请用add_gallery
        :param name:
        :return:
        """
        self.adb.broadcast(self.action, {"api": "SCAN_ALBUM", "name": name})


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# Created by xiazeng on 2019-06-18
# import unittest
import json
import os
from minium.framework.libs import unittest
from .miniconfig import MiniConfig, get_log_level
from .assertbase import AssertBase

FILENAME_SUMMARY = "summary.json"


class MiniResult(unittest.TestResult):
    def __init__(self, stream=None, descriptions=None, verbosity=None):
        super(MiniResult, self).__init__()
        self.__stop_when_framework_error = False
        self.error_stages = []
        self.summary = {
            "test_num": 0,
            "errors": [],
            "failures": [],
            "class_errors": [],
            "class_failures": [],
        }

    def setStopWhenFrameworkError(self, value: bool):
        self.__stop_when_framework_error = bool(value)

    def startTest(self, test: AssertBase) -> None:
        _startTest = getattr(test, "_startTest", None)
        if _startTest:
            setattr(test, "__skip_mini_init__", True)
            _startTest()
        return super().startTest(test)

    def stopTest(self, test: AssertBase) -> None:
        _endTest = getattr(test, "_endTest", None)
        if _endTest:
            _endTest()
        return super().stopTest(test)

    def addError(self, test, err):
        super(MiniResult, self).addError(test, err)
        if getattr(test, "_testMethodName", None) and getattr(
            getattr(test, test._testMethodName, None), "__stop_when_error", None
        ):
            self.stop()

    def addErrorStage(self, test, err, error_stages):
        self.error_stages.append((
            test, error_stages, err
        ))
        self.addError(test, err)
        if self.__stop_when_framework_error and self.hasFrameworkError(error_stages):
            self.stop()

    def addFailure(self, test: unittest.case.TestCase, err):
        super(MiniResult, self).addFailure(test, err)
        test_method = getattr(test, test._testMethodName)
        if getattr(test_method, "__stop_when_error", None):
            self.stop()

    def hasFrameworkError(self, error_stages):
        for error_stage in error_stages:
            if error_stage.startswith("_mini"):
                return True
        return False

    def getFrameworkError(self):
        """
        有框架引起的错误
        """
        for test, error_stages, err in self.error_stages:
            if self.hasFrameworkError(error_stages):
                return err
        return None

    def finish(self):
        self.summary["test_num"] = self.testsRun

        def format_error_info(_error):
            test_case = _error[0]
            if isinstance(test_case, unittest.TestCase):
                return {
                    "module": test_case.__class__.__name__,
                    "error_type": "test_case",
                    "case_name": test_case._testMethodName,
                    "exception": _error[1],
                }
            else:
                print("setupClass or tearDownClass failed")
                return {
                    "error_type": "test_class",
                    "exception": _error[1],
                    "description": test_case.description,
                }

        for error in self.errors:
            format_info = format_error_info(error)
            if format_info["error_type"] == "test_case":
                self.summary["errors"].append(format_info)
            elif format_info["error_type"] == "test_class":
                self.summary["class_errors"].append(format_info)

        for failure in self.failures:
            format_info = format_error_info(failure)
            if format_info["error_type"] == "test_case":
                self.summary["failures"].append(format_info)
            elif format_info["error_type"] == "test_class":
                self.summary["class_failures"].append(format_info)

    def print_shot_msg(self):
        self.finish()
        title = (
            f"case num:{self.summary['test_num']}, "
            f"failed num:{len(self.summary['failures'])}, "
            f"error num:{len(self.summary['errors'])}"
        )
        title = "=" * 20 + title + "=" * 20
        print(title)
        for error in self.summary["errors"]:
            print(f"{error['module']}:{error['case_name']} has error:")
            print(error["exception"])
            print("-" * len(title))
        for error in self.summary["failures"]:
            print(f"{error['module']}:{error['case_name']} has failure:")
            print(error["exception"])
            print("-" * len(title))

        for error in self.summary["class_errors"]:
            print(f"{error['description']} has class error:")
            print(error["exception"])
            print("-" * len(title))
        for error in self.summary["class_failures"]:
            print(f"{error['description']} has class failure:")
            print(error["exception"])
            print("-" * len(title))

    def dumps(self, output_dir):
        self.finish()
        summary_path = os.path.join(output_dir, FILENAME_SUMMARY)
        if os.path.isfile(summary_path):
            with open(summary_path, "r", encoding="utf8") as fd:
                _summary = json.loads(fd.read() or "{}")
        else:
            _summary = {}
        _summary.update(self.summary)
        with open(summary_path, "w", encoding="utf8") as fd:
            json.dump(_summary, fd, indent=4)


class Test1(unittest.TestCase):
    def test_1(self):
        self.assertTrue(True)

    def test_2(self):
        self.assertTrue(False, "failed")


class Test2(unittest.TestCase):
    def setUp(self):
        raise RuntimeError("setup failed")

    def test_1(self):
        self.assertTrue(True)

    def test_2(self):
        self.assertTrue(False, "failed")


class Test3(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        raise RuntimeError("setup class failed")

    def test_1(self):
        self.assertTrue(True)

    def test_2(self):
        self.assertTrue(False, "failed")


class Test4(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        raise RuntimeError("setup class failed")

    def test_1(self):
        self.assertTrue(True)

    def test_2(self):
        self.assertTrue(False, "failed")


if __name__ == "__main__":
    loader = unittest.TestLoader()
    tests = loader.loadTestsFromTestCase(Test1)
    tests1 = loader.loadTestsFromTestCase(Test2)
    tests2 = loader.loadTestsFromTestCase(Test3)
    tests3 = loader.loadTestsFromTestCase(Test4)
    tests.addTests(tests1)
    tests.addTests(tests2)
    tests.addTests(tests3)
    result = MiniResult()
    tests.run(result)
    result.print_shot_msg()



#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         lockerzhang
Filename:       minium_log.py
Create time:    2019-08-29 11:12
Description:

"""

from functools import wraps
import datetime
import types
import json
import requests
import queue
import threading
import logging
import os
import time
import traceback
from .version import build_version
from ...framework.exception import MiniError

logging.getLogger("urllib3").setLevel(logging.WARNING)
# logger = logging.getLogger("DataReport")

REPORT_DOMAIN = "minitest.weixin.qq.com"
REPORT_PATH = "xbeacon/user_report"
app_id = None
version = build_version().get("version")
revision = build_version().get("revision")

class ReportData(dict):
    cmd = ""

    def dumps(self):
        return json.dumps(self)

class ExceptionData(dict):
    def __init__(self, err: Exception, **kwargs):
        kwargs["from"] = "minium"
        self.TimeStamp = getattr(err, "timestamp", None) or time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        self.error = err.__class__.__name__
        self.err_msg = getattr(err, "msg", None) or str(err)
        self.Uin = 0
        self.version = version
        self.revision = revision
        self.ext = ""
        self.app_id = app_id
        self.__dict__.update(kwargs)
        super().__init__(self.__dict__)
        self.cmd = "cloud_exception_log"

class ConsumeData(dict):
    def __init__(self, __map: dict, **kwargs):
        self.version = version
        self.revision = revision
        self.app_id = app_id
        self.time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        __map.update(kwargs)
        self.__dict__.update(__map)
        super().__init__(self.__dict__)
        self.cmd = "api_log"

def process_report():
    global existFlag
    while not existFlag:
        lock.acquire()
        lock.wait(10)
        lock.release()
        if not report_queue.empty():
            data = report_queue.get()
            # logger.debug("Thread processing report data %s" % data)
            report(data=data)


def report_fail(failed=True):
    global fail, existFlag
    if failed:
        fail += 1
        if fail >= 10:
            existFlag = 1
    else:
        fail = 0


def report_exception(data: ExceptionData):
    if not data.app_id:
        return
    report_queue.put(data)
    lock.acquire()
    lock.notify()
    lock.release()


def report(data: ReportData):
    """
    report minium_new
    """
    if existFlag:
        return
    print(f"https://{REPORT_DOMAIN}/{REPORT_PATH}/{data.cmd}")
    print(data)
    try:
        ret = requests.post(
            url=f"https://{REPORT_DOMAIN}/{REPORT_PATH}/{data.cmd}",
            data=data.dumps(),
            timeout=10,
        )
        # logger.debug(ret.text)
        report_fail(ret.status_code != 200)
    except Exception as e:
        # logger.debug("data report fail with https")
        try:
            ret = requests.post(
                url=f"http://{REPORT_DOMAIN}/{REPORT_PATH}/{data.cmd}",
                data=data.dumps(),
                timeout=10,
            )
            # logger.debug(ret.text)
            report_fail(ret.status_code != 200)
        except Exception as e:
            # logger.error("data report fail with http, give up")
            # logger.exception(e)
            pass


# logger.debug(ret.text)


existFlag = 0
fail = 0

lock = threading.Condition()
report_queue = queue.Queue()
thread = threading.Thread(target=process_report)
thread.setDaemon(True)
thread.start()

usage = []

REPORT_THRESHOLD = 10000  # 上报阈值

def minium_log(func):
    """
    函数统计装饰器
    :param func:
    :return:
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        global usage, app_id

        start = datetime.datetime.now()
        self = args[0]
        func_name = f"{self.__class__.__name__}.{func.__name__}"
        try:
            result = func(*args, **kwargs)
        except MiniError as me:
            report_exception(ExceptionData(me, func=func_name))
            raise
        end = datetime.datetime.now()
        consuming = int((end - start).total_seconds() * 1000)
        if app_id is None and hasattr(self, "app_id"):
            app_id = self.app_id
        if consuming < REPORT_THRESHOLD:
            return result
        new_args = list(args[1:])
        
        consum_data = ConsumeData({
            "func": func_name,
            "args": str(new_args),
            "kwargs": kwargs,
            "consuming": consuming
        })

        if app_id is None:
            usage.append(consum_data)
        else:
            report_queue.put(consum_data)
            for f in usage:
                f["app_id"] = app_id
                report_queue.put(f)
            usage.clear()
            lock.acquire()
            lock.notify()
            lock.release()
        return result

    return wrapper


class MonitorMetaClass(type):
    """
    类监控元类
    """

    def __new__(mcs, cls_name, bases, attr_dict):
        for k, v in attr_dict.items():
            if (
                isinstance(v, types.FunctionType)
                and not k.startswith("_")
                and not k.startswith("send")
                and not k.startswith("register")
                and not k.startswith("notify")
                and not k.startswith("remove")
            ):
                attr_dict[k] = minium_log(v)
        return type.__new__(mcs, cls_name, bases, attr_dict)




#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         lockerzhang
Filename:       minisuite.py
Create time:    2020/1/3 16:22
Update time:    2022/4/12 16:29
Description:    测试suite实例, 同时记录package的import状态

"""
import os
import json
import fnmatch
import logging.handlers
from .exception import MiniParamsError
import json
from json import JSONDecodeError
import traceback

logger = logging.getLogger("minium")


class MiniSuite(object):
    def __init__(self, path_or_dict=None):
        self.__suite_json = {}
        if isinstance(path_or_dict, (str, bytes)):
            try:
                if os.path.exists(path_or_dict):
                    with open(path_or_dict, "rb") as fd:
                        content = fd.read()
                    try:
                        self.__suite_json = json.loads(content.decode("utf8"))  # use utf8 encoding
                    except UnicodeDecodeError:
                        self.__suite_json = json.loads(content.decode("gbk"))
                else:
                    self.__suite_json = json.loads(path_or_dict)
            except JSONDecodeError as e:
                raise MiniParamsError(f"Suite param {path_or_dict} not file neither dict or formatted json string: {e}") from e
        elif isinstance(path_or_dict, dict):
            self.__suite_json = path_or_dict
        else:
            raise MiniParamsError(f"Suite param should string or dict, not {type(path_or_dict).__name__}")
        self.__success_pkg = set()
        self.__fail_pkg = {}

    def __getattribute__(self, __name: str):
        try:
            return super().__getattribute__(__name)
        except AttributeError:
            if __name in self.__suite_json:
                return self.__suite_json.get(__name)
            raise

    def get_case_config(self, cls_name, case_name):
        for pkg_cases in self.pkg_list:
            pkg = pkg_cases["pkg"]
            if fnmatch.fnmatch(cls_name, pkg):
                for case_info in pkg["case_list"]:
                    if isinstance(case_info, dict):
                        if fnmatch.fnmatch(case_info["name"], case_name):
                            return case_info.get("attr", dict())
        return dict()

    @property
    def pkg_list(self):
        return self.__suite_json.get("pkg_list", list())

    @property
    def case_list(self):
        return self.__suite_json.get("case_list", list())

    def set_pkg_success(self, pkg: str):
        self.__success_pkg.add(pkg)

    def set_pkg_fail(self, pkg: str, e: Exception = None):
        if pkg not in self.__fail_pkg:
            stack_lines = traceback.format_exception(type(e), e, e.__traceback__)
            self.__fail_pkg[pkg] = {
                "error_type": type(e).__name__,
                "error_value": str(e.args[0]) if len(e.args) > 0 else "",
                "errors": "".join(stack_lines)
            }

    @property
    def success_pkg(self):
        return list(self.__success_pkg)

    @property
    def fail_pkg(self):
        return dict(self.__fail_pkg)

    def get_pkg_import_result(self):
        return {
            "success": self.success_pkg,
            "fail": self.fail_pkg,
        }
